# -*- coding: utf-8 -*-
"""Windows EventLog resources database reader."""

import collections
import os
import sqlite3

from acstore import sqlite_store
from acstore.containers import interface as containers_interface

from plaso.engine import path_helper
from plaso.helpers.windows import languages
from plaso.helpers.windows import resource_files
from plaso.output import logger


class Sqlite3DatabaseFile(object):
  """Class that defines a sqlite3 database file."""

  _HAS_TABLE_QUERY = (
      'SELECT name FROM sqlite_master '
      'WHERE type = "table" AND name = "{0:s}"')

  def __init__(self):
    """Initializes the database file object."""
    super(Sqlite3DatabaseFile, self).__init__()
    self._connection = None
    self._cursor = None
    self.filename = None
    self.read_only = None

  def Close(self):
    """Closes the database file.

    Raises:
      RuntimeError: if the database is not opened.
    """
    if not self._connection:
      raise RuntimeError('Cannot close database not opened.')

    # We need to run commit or not all data is stored in the database.
    self._connection.commit()
    self._connection.close()

    self._connection = None
    self._cursor = None
    self.filename = None
    self.read_only = None

  def HasTable(self, table_name):
    """Determines if a specific table exists.

    Args:
      table_name (str): table name.

    Returns:
      bool: True if the table exists.

    Raises:
      RuntimeError: if the database is not opened.
    """
    if not self._connection:
      raise RuntimeError(
          'Cannot determine if table exists database not opened.')

    sql_query = self._HAS_TABLE_QUERY.format(table_name)

    self._cursor.execute(sql_query)
    if self._cursor.fetchone():
      return True

    return False

  def GetValues(self, table_names, column_names, condition):
    """Retrieves values from a table.

    Args:
      table_names (list[str]): table names.
      column_names (list[str]): column names.
      condition (str): query condition such as
          "log_source == 'Application Error'".

    Yields:
      sqlite3.row: row.

    Raises:
      RuntimeError: if the database is not opened.
    """
    if not self._connection:
      raise RuntimeError('Cannot retrieve values database not opened.')

    if condition:
      condition = f' WHERE {condition:s}'
    else:
      condition = ''

    table_names_string = ', '.join(table_names)
    column_names_string = ', '.join(column_names)
    sql_query = (
        f'SELECT {column_names_string:s} FROM {table_names_string:s}'
        f'{condition:s}')

    self._cursor.execute(sql_query)

    # TODO: have a look at https://docs.python.org/2/library/
    # sqlite3.html#sqlite3.Row.
    for row in self._cursor:
      yield {
          column_name: row[column_index]
          for column_index, column_name in enumerate(column_names)}

  def Open(self, filename, read_only=False):
    """Opens the database file.

    Args:
      filename (str): filename of the database.
      read_only (Optional[bool]): True if the database should be opened in
          read-only mode. Since sqlite3 does not support a real read-only
          mode we fake it by only permitting SELECT queries.

    Returns:
      bool: True if successful.

    Raises:
      RuntimeError: if the database is already opened.
    """
    if self._connection:
      raise RuntimeError('Cannot open database already opened.')

    self.filename = filename
    self.read_only = read_only

    try:
      self._connection = sqlite3.connect(filename)
    except sqlite3.OperationalError:
      return False

    if not self._connection:
      return False

    self._cursor = self._connection.cursor()
    if not self._cursor:
      return False

    return True


class WinevtResourcesSqlite3DatabaseReader(object):
  """Windows EventLog resources SQLite database reader."""

  def __init__(self):
    """Initializes a Windows EventLog resources SQLite database reader."""
    super(WinevtResourcesSqlite3DatabaseReader, self).__init__()
    self._database_file = Sqlite3DatabaseFile()
    self._resouce_file_helper = resource_files.WindowsResourceFileHelper
    self._string_format = 'wrc'

  def _GetEventLogProviderKey(self, log_source):
    """Retrieves the EventLog provider key.

    Args:
      log_source (str): EventLog source.

    Returns:
      str: EventLog provider key or None if not available.

    Raises:
      RuntimeError: if more than one value is found in the database.
    """
    table_names = ['event_log_providers']
    column_names = ['event_log_provider_key']
    condition = f'log_source == "{log_source:s}"'

    values_list = list(self._database_file.GetValues(
        table_names, column_names, condition))

    number_of_values = len(values_list)
    if number_of_values == 0:
      return None

    if number_of_values == 1:
      values = values_list[0]
      return values['event_log_provider_key']

    raise RuntimeError('More than one value found in database.')

  def _GetMessage(self, message_file_key, lcid, message_identifier):
    """Retrieves a specific message from a specific message table.

    Args:
      message_file_key (int): message file key.
      lcid (int): language code identifier (LCID).
      message_identifier (int): message identifier.

    Returns:
      str: message string or None if not available.

    Raises:
      RuntimeError: if more than one value is found in the database.
    """
    table_name = f'message_table_{message_file_key:d}_0x{lcid:08x}'

    has_table = self._database_file.HasTable(table_name)
    if not has_table:
      return None

    column_names = ['message_string']
    condition = f'message_identifier == "0x{message_identifier:08x}"'

    values = list(self._database_file.GetValues(
        [table_name], column_names, condition))

    number_of_values = len(values)
    if number_of_values == 0:
      return None

    if number_of_values == 1:
      return values[0]['message_string']

    raise RuntimeError('More than one value found in database.')

  def _GetMessageFileKeys(self, event_log_provider_key):
    """Retrieves the message file keys.

    Args:
      event_log_provider_key (int): EventLog provider key.

    Yields:
      int: message file key.
    """
    table_names = ['message_file_per_event_log_provider']
    column_names = ['message_file_key']
    condition = f'event_log_provider_key == {event_log_provider_key:d}'

    generator = self._database_file.GetValues(
        table_names, column_names, condition)
    for values in generator:
      yield values['message_file_key']

  def Close(self):
    """Closes the database reader object."""
    self._database_file.Close()

  def GetMessage(self, log_source, lcid, message_identifier):
    """Retrieves a specific message for a specific EventLog source.

    Args:
      log_source (str): EventLog source, such as "Application Error".
      lcid (int): language code identifier (LCID).
      message_identifier (int): message identifier.

    Returns:
      str: message string or None if not available.
    """
    event_log_provider_key = self._GetEventLogProviderKey(log_source)
    if not event_log_provider_key:
      return None

    generator = self._GetMessageFileKeys(event_log_provider_key)
    if not generator:
      return None

    message_string = None
    for message_file_key in generator:
      message_string = self._GetMessage(
          message_file_key, lcid, message_identifier)

      if message_string:
        break

    if self._string_format == 'wrc':
      message_string = self._resouce_file_helper.FormatMessageStringInPEP3101(
          message_string)

    return message_string

  def GetMetadataAttribute(self, attribute_name):
    """Retrieves the metadata attribute.

    Args:
      attribute_name (str): name of the metadata attribute.

    Returns:
      str: the metadata attribute or None.

    Raises:
      RuntimeError: if more than one value is found in the database.
    """
    table_name = 'metadata'

    has_table = self._database_file.HasTable(table_name)
    if not has_table:
      return None

    column_names = ['value']
    condition = f'name == "{attribute_name:s}"'

    values = list(self._database_file.GetValues(
        [table_name], column_names, condition))

    number_of_values = len(values)
    if number_of_values == 0:
      return None

    if number_of_values == 1:
      return values[0]['value']

    raise RuntimeError('More than one value found in database.')

  def Open(self, filename):
    """Opens the database reader object.

    Args:
      filename (str): filename of the database.

    Returns:
      bool: True if successful.

    Raises:
      RuntimeError: if the version or string format of the database
          is not supported.
    """
    if not self._database_file.Open(filename, read_only=True):
      return False

    version = self.GetMetadataAttribute('version')
    if not version or version != '20150315':
      raise RuntimeError(f'Unsupported version: {version:s}')

    string_format = self.GetMetadataAttribute('string_format')
    if not string_format:
      string_format = 'wrc'

    if string_format not in ('pep3101', 'wrc'):
      raise RuntimeError(f'Unsupported string format: {string_format:s}')

    self._string_format = string_format
    return True


class WinevtResourcesEventLogProvider(containers_interface.AttributeContainer):
  """Windows Event Log provider.

  Attributes:
    additional_identifier (str): additional identifier of the provider,
        contains a GUID.
    category_message_files (set[str]): paths of the category message files.
    event_message_files (set[str]): paths of the event message files.
    identifier (str): identifier of the provider, contains a GUID.
    log_sources (list[str]): names of the corresponding Event Log sources.
    log_types (list[str]): Windows Event Log types.
    name (str): name of the provider.
    parameter_message_files (set[str]): paths of the parameter message
        files.
    windows_version (str): Windows version.
  """

  CONTAINER_TYPE = 'winevtrc_eventlog_provider'

  SCHEMA = {
      'additional_identifier': 'str',
      'category_message_files': 'List[str]',
      'event_message_files': 'List[str]',
      'identifier': 'str',
      'log_sources': 'List[str]',
      'log_types': 'List[str]',
      'name': 'str',
      'parameter_message_files': 'List[str]',
      'windows_version': 'str'}

  def __init__(self):
    """Initializes the Windows Event Log provider."""
    super(WinevtResourcesEventLogProvider, self).__init__()
    self.additional_identifier = None
    self.category_message_files = set()
    self.event_message_files = set()
    self.identifier = None
    self.log_sources = []
    self.log_types = []
    self.name = None
    self.parameter_message_files = set()
    self.windows_version = None


class WinevtResourcesMessageFile(containers_interface.AttributeContainer):
  """Windows Event Log message file.

  Attributes:
    file_version (str): file version.
    product_version (str): product version.
    windows_path (str): path as defined by the Window Event Log provider.
    windows_version (str): Windows version.
  """

  CONTAINER_TYPE = 'winevtrc_message_file'

  SCHEMA = {
      'file_version': 'str',
      'product_version': 'str',
      'windows_path': 'str',
      'windows_version': 'str'}

  def __init__(
      self, file_version=None, product_version=None, windows_path=None,
      windows_version=None):
    """Initializes a Windows Event Log message file.

    Args:
      file_version (Optional[str]): file version.
      product_version (Optional[str]): product version.
      windows_path (Optional[str]): path as defined by the Window Event Log
          provider.
      windows_version (Optional[str]): Windows version.
    """
    super(WinevtResourcesMessageFile, self).__init__()
    self.file_version = file_version
    self.product_version = product_version
    self.windows_path = windows_path
    self.windows_version = windows_version


class WinevtResourcesMessageString(containers_interface.AttributeContainer):
  """Windows Event Log message string.

  Attributes:
    identifier (int): message identifier.
    text (str): message text.
  """

  CONTAINER_TYPE = 'winevtrc_message_string'

  SCHEMA = {
      '_message_table_identifier': 'AttributeContainerIdentifier',
      'language_identifier': 'str',
      'message_identifier': 'int',
      'text': 'str'}

  _SERIALIZABLE_PROTECTED_ATTRIBUTES = [
      '_message_table_identifier']

  def __init__(self, message_identifier=None, text=None):
    """Initializes a Windows Event Log message string.

    Args:
      message_identifier (Optional[int]): message identifier.
      text (Optional[int]): message text.
    """
    super(WinevtResourcesMessageString, self).__init__()
    self._message_table_identifier = None
    self.message_identifier = message_identifier
    self.text = text

  def GetMessageTableIdentifier(self):
    """Retrieves the identifier of the associated message table.

    Returns:
      AttributeContainerIdentifier: message table identifier or None when not
          set.
    """
    return self._message_table_identifier

  def SetMessageTableIdentifier(self, message_table_identifier):
    """Sets the identifier of the associated message table.

    Args:
      message_table_identifier (AttributeContainerIdentifier): message table
          identifier.
    """
    self._message_table_identifier = message_table_identifier


class WinevtResourcesMessageStringMapping(
    containers_interface.AttributeContainer):
  """Windows Event Log message string mapping.

  Attributes:
    event_identifier (int): event identifier.
    event_version (int): event version.
    message_identifier (int): message identifier.
    provider_identifier (str): Event Log provider identifier.
  """

  CONTAINER_TYPE = 'winevtrc_message_string_mapping'

  SCHEMA = {
      '_message_file_identifier': 'AttributeContainerIdentifier',
      'event_identifier': 'int',
      'event_version': 'int',
      'message_identifier': 'int',
      'provider_identifier': 'str'}

  _SERIALIZABLE_PROTECTED_ATTRIBUTES = [
      '_message_file_identifier']

  def __init__(
      self, event_identifier=None, event_version=None, message_identifier=None,
      provider_identifier=None):
    """Initializes a Windows Event Log message string mapping.

    Args:
      event_identifier (Optional[int]): event identifier.
      event_version (Optional[int]): event version.
      message_identifier (Optional[int]): message identifier.
      provider_identifier (Optional[str]): Event Log provider identifier.
    """
    super(WinevtResourcesMessageStringMapping, self).__init__()
    self._message_file_identifier = None
    self.event_identifier = event_identifier
    self.event_version = event_version
    self.message_identifier = message_identifier
    self.provider_identifier = provider_identifier

  def GetMessageFileIdentifier(self):
    """Retrieves the identifier of the associated message file.

    Returns:
      AttributeContainerIdentifier: message file identifier or None when
          not set.
    """
    return self._message_file_identifier

  def SetMessageFileIdentifier(self, message_file_identifier):
    """Sets the identifier of the associated message file.

    Args:
      message_file_identifier (AttributeContainerIdentifier): message file
          identifier.
    """
    self._message_file_identifier = message_file_identifier


class WinevtResourcesMessageTable(containers_interface.AttributeContainer):
  """Windows Event Log message table.

  Attributes:
    language_identifier (int): language identifier (LCID).
  """

  CONTAINER_TYPE = 'winevtrc_message_table'

  SCHEMA = {
      '_message_file_identifier': 'AttributeContainerIdentifier',
      'language_identifier': 'int'}

  _SERIALIZABLE_PROTECTED_ATTRIBUTES = [
      '_message_file_identifier']

  def __init__(self, language_identifier=None):
    """Initializes a Windows Event Log message table descriptor.

    Args:
      language_identifier (Optional[int]): language identifier (LCID).
    """
    super(WinevtResourcesMessageTable, self).__init__()
    self._message_file_identifier = None
    self.language_identifier = language_identifier

  def GetMessageFileIdentifier(self):
    """Retrieves the identifier of the associated message file.

    Returns:
      AttributeContainerIdentifier: message file identifier or None when not
          set.
    """
    return self._message_file_identifier

  def SetMessageFileIdentifier(self, message_file_identifier):
    """Sets the identifier of the associated message file.

    Args:
      message_file_identifier (AttributeContainerIdentifier): message file
          identifier.
    """
    self._message_file_identifier = message_file_identifier


class WinevtResourcesAttributeContainerStore(
    sqlite_store.SQLiteAttributeContainerStore):
  """Windows EventLog resources attribute container store.

  Attributes:
    format_version (int): storage format version.
    serialization_format (str): serialization format.
    string_format (str): string format.
  """

  _FORMAT_VERSION = 20240929
  _APPEND_COMPATIBLE_FORMAT_VERSION = 20240929
  _UPGRADE_COMPATIBLE_FORMAT_VERSION = 20240929
  _READ_COMPATIBLE_FORMAT_VERSION = 20240929

  def __init__(self, string_format='wrc'):
    """Initializes a message resource attribute container store.

    Args:
      string_format (Optional[str]): string format. The default is the Windows
          Resource (wrc) format.
    """
    super(WinevtResourcesAttributeContainerStore, self).__init__()
    self.string_format = string_format

    self._containers_manager.RegisterAttributeContainers([
        WinevtResourcesEventLogProvider, WinevtResourcesMessageFile,
        WinevtResourcesMessageString, WinevtResourcesMessageStringMapping,
        WinevtResourcesMessageTable])

  def _ReadAndCheckStorageMetadata(self, check_readable_only=False):
    """Reads storage metadata and checks that the values are valid.

    Args:
      check_readable_only (Optional[bool]): whether the store should only be
          checked to see if it can be read. If False, the store will be checked
          to see if it can be read and written to.

    Raises:
      IOError: when there is an error querying the attribute container store.
      OSError: when there is an error querying the attribute container store.
    """
    metadata_values = self._ReadMetadata()

    self._CheckStorageMetadata(
        metadata_values, check_readable_only=check_readable_only)

    string_format = metadata_values.get('string_format', None)

    if string_format not in ('pep3101', 'wrc'):
      raise IOError(f'Unsupported string format: {string_format:s}')

    self.format_version = metadata_values['format_version']
    self.serialization_format = metadata_values['serialization_format']
    self.string_format = metadata_values['string_format']


class WinevtResourcesHelper(object):
  """Windows EventLog resources helper."""

  # LCID 0x0409 is en-US.
  DEFAULT_LCID = 0x0409

  _DEFAULT_PARAMETER_MESSAGE_FILES = (
      '%SystemRoot%\\System32\\MsObjs.dll',
      '%SystemRoot%\\System32\\kernel32.dll')

  # The maximum number of cached message strings
  _MAXIMUM_CACHED_MESSAGE_STRINGS = 64 * 1024

  _WINEVT_RC_DATABASE = 'winevt-rc.db'

  def __init__(self, storage_reader, data_location, lcid):
    """Initializes Windows EventLog resources helper.

    Args:
      storage_reader (StorageReader): storage reader.
      data_location (str): data location of the winevt-rc database.
      lcid (int): Windows Language Code Identifier (LCID).
    """
    language_tag = languages.WindowsLanguageHelper.GetLanguageTagForLCID(
        lcid or self.DEFAULT_LCID)

    super(WinevtResourcesHelper, self).__init__()
    self._data_location = data_location
    self._environment_variables = None
    self._language_tag = language_tag.lower()
    self._lcid = lcid or self.DEFAULT_LCID
    self._message_string_cache = collections.OrderedDict()
    self._resouce_file_helper = resource_files.WindowsResourceFileHelper
    self._storage_reader = None
    self._windows_eventlog_message_files = None
    self._windows_eventlog_providers = None
    self._winevt_database_reader = None

    if storage_reader and storage_reader.HasAttributeContainers(
        'windows_eventlog_provider'):
      self._storage_reader = storage_reader

  def _CacheMessageString(
      self, provider_identifier, log_source, message_identifier,
      event_version, message_string):
    """Caches a specific message string.

    Args:
      provider_identifier (str): EventLog provider identifier.
      log_source (str): EventLog source, such as "Application Error".
      message_identifier (int): message identifier.
      event_version (int): event version or None if not set.
      message_string (str): message string.
    """
    if len(self._message_string_cache) >= self._MAXIMUM_CACHED_MESSAGE_STRINGS:
      self._message_string_cache.popitem(last=True)

    if provider_identifier:
      lookup_key = f'{provider_identifier:s}:0x{message_identifier:08x}'
      if event_version is not None:
        lookup_key = f'{lookup_key:s}:{event_version:d}'
      self._message_string_cache[lookup_key] = message_string
      self._message_string_cache.move_to_end(lookup_key, last=False)

    if log_source:
      lookup_key = f'{log_source:s}:0x{message_identifier:08x}'
      if event_version is not None:
        lookup_key = f'{lookup_key:s}:{event_version:d}'
      self._message_string_cache[lookup_key] = message_string
      self._message_string_cache.move_to_end(lookup_key, last=False)

  def _GetCachedMessageString(
      self, provider_identifier, log_source, message_identifier, event_version):
    """Retrieves a specific cached message string.

    Args:
      provider_identifier (str): EventLog provider identifier.
      log_source (str): EventLog source, such as "Application Error".
      message_identifier (int): message identifier.
      event_version (int): event version or None if not set.

    Returns:
      str: message string or None if not available.
    """
    lookup_key = None
    message_string = None

    if provider_identifier:
      lookup_key = f'{provider_identifier:s}:0x{message_identifier:08x}'
      if event_version is not None:
        lookup_key = f'{lookup_key:s}:{event_version:d}'
      message_string = self._message_string_cache.get(lookup_key, None)

    if not message_string and log_source:
      lookup_key = f'{log_source:s}:0x{message_identifier:08x}'
      if event_version is not None:
        lookup_key = f'{lookup_key:s}:{event_version:d}'
      message_string = self._message_string_cache.get(lookup_key, None)

    if message_string:
      self._message_string_cache.move_to_end(lookup_key, last=False)

    return message_string

  def _GetEventMessageFileIdentifiers(self, message_files):
    """Retrieves event message file identifiers.

    Args:
      message_files (list[str]): Windows EventLog message files.

    Returns:
      list[str]: message file identifiers.
    """
    message_file_identifiers = []
    for windows_path in message_files or []:
      path, filename = path_helper.PathHelper.GetWindowsSystemPath(
          windows_path, self._environment_variables)

      lookup_path = '\\'.join([path, filename]).lower()
      message_file_identifier = self._windows_eventlog_message_files.get(
          lookup_path, None)
      if message_file_identifier:
        message_file_identifier = message_file_identifier.CopyToString()
        message_file_identifiers.append(message_file_identifier)

      mui_filename = f'{filename:s}.mui'
      lookup_path = '\\'.join([path, self._language_tag, mui_filename]).lower()
      message_file_identifier = self._windows_eventlog_message_files.get(
          lookup_path, None)
      if message_file_identifier:
        message_file_identifier = message_file_identifier.CopyToString()
        message_file_identifiers.append(message_file_identifier)

    return message_file_identifiers

  def _GetMappedMessageIdentifier(
      self, storage_reader, provider_identifier, message_identifier,
      event_version):
    """Retrieves a WEVT_TEMPLATE mapped message identifier if available.

    Args:
      storage_reader (StorageReader): storage reader.
      provider_identifier (str): EventLog provider identifier.
      message_identifier (int): message identifier.
      event_version (int): event version or None if not set.

    Returns:
      int: message identifier.
    """
    # Map the event identifier to a message identifier as defined by the
    # WEVT_TEMPLATE event definition.
    if provider_identifier and storage_reader.HasAttributeContainers(
        'windows_wevt_template_event'):
      # TODO: add message_file_identifiers to filter_expression
      filter_expression = (
          f'provider_identifier == "{provider_identifier:s}" and '
          f'identifier == {message_identifier:d}')
      if event_version is not None:
        filter_expression = (
            f'{filter_expression:s} and version == {event_version:d}')

      for event_definition in storage_reader.GetAttributeContainers(
          'windows_wevt_template_event', filter_expression=filter_expression):
        logger.debug((
            f'Message: 0x{message_identifier:08x} of provider: '
            f'{provider_identifier:s} maps to: '
            f'0x{event_definition.message_identifier:08x}'))

        return event_definition.message_identifier

    return message_identifier

  def _GetMessageStrings(
      self, storage_reader, message_file_identifiers, message_identifier):
    """Retrieves message strings.

    Args:
      storage_reader (StorageReader): storage reader.
      message_file_identifiers (list[str]): message file identifiers.
      message_identifier (int): message identifier.

    Returns:
      list[str]: message strings.
    """
    message_strings = []

    # TODO: add message_file_identifiers to filter_expression
    filter_expression = (
        f'language_identifier == {self._lcid:d} and '
        f'message_identifier == {message_identifier:d}')

    for message_string in storage_reader.GetAttributeContainers(
        'windows_eventlog_message_string', filter_expression=filter_expression):
      identifier = message_string.GetMessageFileIdentifier()
      identifier = identifier.CopyToString()
      if identifier in message_file_identifiers:
        message_strings.append(message_string)

    return message_strings

  def _GetMessageStringsWithMessageTable(
      self, storage_reader, message_file_identifiers, message_identifier):
    """Retrieves message strings.

    Args:
      storage_reader (StorageReader): storage reader.
      message_file_identifiers (list[str]): message file identifiers.
      message_identifier (int): message identifier.

    Returns:
      list[str]: message strings.
    """
    message_strings = []

    for message_file_identifier in message_file_identifiers:
      filter_expression = (
          f'_message_file_identifier == "{message_file_identifier:s}" and '
          f'language_identifier == {self._lcid:d}')
      for message_table in storage_reader.GetAttributeContainers(
          'winevtrc_message_table', filter_expression=filter_expression):
        if not message_table:
          continue

        identifier = message_table.GetIdentifier()
        message_table_identifier = identifier.CopyToString()

        filter_expression = (
            f'_message_table_identifier == "{message_table_identifier:s}" and '
            f'message_identifier == {message_identifier:d}')

        identifier = message_table.GetMessageFileIdentifier()
        message_file_identifier = identifier.CopyToString()

        for message_string in storage_reader.GetAttributeContainers(
            'winevtrc_message_string', filter_expression=filter_expression):
          if message_file_identifier in message_file_identifiers:
            message_strings.append(message_string)

    return message_strings

  def _GetWindowsEventLogProvider(self, provider_identifier, log_source):
    """Retrieves a Windows EventLog provider.

    Args:
      provider_identifier (str): EventLog provider identifier.
      log_source (str): EventLog source, such as "Application Error".

    Returns:
      tuple[WindowsEventLogProviderArtifact, str]: Windows EventLog provider
          or None if not available, and provider lookup key.
    """
    provider = None

    if provider_identifier:
      lookup_key = provider_identifier.lower()
      provider = self._windows_eventlog_providers.get(lookup_key, None)

    if not provider:
      lookup_key = log_source.lower()
      provider = self._windows_eventlog_providers.get(lookup_key, None)

    return provider, lookup_key

  def _GetWinevtRcDatabaseReader(self):
    """Opens the Windows EventLog resource database reader.

    Returns:
      WinevtResourcesSqlite3DatabaseReader: Windows EventLog resource
          database reader or None.
    """
    if not self._winevt_database_reader and self._data_location:
      logger.warning((
          f'Falling back to {self._WINEVT_RC_DATABASE:s}. Please make sure '
          f'the Windows EventLog message strings in the database correspond '
          f'to those in the EventLog files.'))

      database_path = os.path.join(
          self._data_location, self._WINEVT_RC_DATABASE)
      if not os.path.isfile(database_path):
        return None

      try:
        self._winevt_database_reader = WinevtResourcesSqlite3DatabaseReader()
        result = self._winevt_database_reader.Open(database_path)
      except sqlite3.OperationalError:
        result = False

      if not result:
        try:
          self._winevt_database_reader = (
              WinevtResourcesAttributeContainerStore())
          self._winevt_database_reader.Open(path=database_path, read_only=True)  # pylint: disable=no-value-for-parameter,unexpected-keyword-arg
          result = True
        except IOError:
          result = False

      if not result:
        self._winevt_database_reader = None

    return self._winevt_database_reader

  def _GetWinevtRcDatabaseMessageString(
      self, provider_identifier, log_source, message_identifier, event_version):
    """Retrieves a specific Windows EventLog resource database message string.

    Args:
      provider_identifier (str): EventLog provider identifier.
      log_source (str): EventLog source, such as "Application Error".
      message_identifier (int): message identifier.
      event_version (int): event version or None if not set.

    Returns:
      str: message string or None if not available.
    """
    database_reader = self._GetWinevtRcDatabaseReader()
    if not database_reader:
      return None

    if isinstance(database_reader, WinevtResourcesSqlite3DatabaseReader):
      return database_reader.GetMessage(
          log_source, self._lcid, message_identifier)

    if self._windows_eventlog_providers is None:
      self._ReadWindowsEventLogProviders(
          database_reader, container_type='winevtrc_eventlog_provider')

    if self._windows_eventlog_message_files is None:
      self._ReadWindowsEventLogMessageFiles(
          database_reader, container_type='winevtrc_message_file',
          path_attribute='windows_path')

    provider, provider_lookup_key = self._GetWindowsEventLogProvider(
        provider_identifier, log_source)
    if not provider:
      return None

    original_message_identifier = message_identifier

    # TODO: pass message_file_identifiers
    message_identifier = self._GetMappedMessageIdentifier(
        database_reader, provider_identifier, message_identifier, event_version)

    message_file_identifiers = self._GetEventMessageFileIdentifiers(
        provider.event_message_files)
    if not message_file_identifiers:
      logger.warning((
          f'No event message file for identifier: 0x{message_identifier:08x} '
          f'(original: 0x{original_message_identifier:08x}) '
          f'of provider: {provider_lookup_key:s}'))
      return None

    message_strings = self._GetMessageStringsWithMessageTable(
        database_reader, message_file_identifiers, message_identifier)
    if not message_strings:
      logger.warning((
          f'No message string for identifier: 0x{message_identifier:08x} '
          f'(original: 0x{original_message_identifier:08x}) '
          f'of provider: {provider_lookup_key:s}'))
      return None

    message_string = message_strings[0].text
    if database_reader.string_format == 'wrc':
      message_string = self._resouce_file_helper.FormatMessageStringInPEP3101(
          message_string)

    return message_string

  def _ReadEnvironmentVariables(self, storage_reader):
    """Reads the environment variables.

    Args:
      storage_reader (StorageReader): storage reader.
    """
    # TODO: get environment variables related to the source.
    self._environment_variables = list(storage_reader.GetAttributeContainers(
        'environment_variable'))

  def _ReadEventMessageString(
      self, storage_reader, provider_identifier, log_source,
      message_identifier, event_version):
    """Reads an event message string.

    Args:
      storage_reader (StorageReader): storage reader.
      provider_identifier (str): EventLog provider identifier.
      log_source (str): EventLog source, such as "Application Error".
      message_identifier (int): message identifier.
      event_version (int): event version or None if not set.

    Returns:
      str: message string or None if not available.
    """
    if self._environment_variables is None:
      self._ReadEnvironmentVariables(storage_reader)

    if self._windows_eventlog_providers is None:
      self._ReadWindowsEventLogProviders(storage_reader)

    if self._windows_eventlog_message_files is None:
      self._ReadWindowsEventLogMessageFiles(storage_reader)

    provider, provider_lookup_key = self._GetWindowsEventLogProvider(
        provider_identifier, log_source)
    if not provider:
      return None

    if not storage_reader.HasAttributeContainers(
        'windows_eventlog_message_string'):
      return None

    original_message_identifier = message_identifier

    # TODO: pass message_file_identifiers
    message_identifier = self._GetMappedMessageIdentifier(
        storage_reader, provider_identifier, message_identifier, event_version)

    message_file_identifiers = self._GetEventMessageFileIdentifiers(
        provider.event_message_files)
    if not message_file_identifiers:
      logger.warning((
          f'No event message file for identifier: 0x{message_identifier:08x} '
          f'(original: 0x{original_message_identifier:08x}) '
          f'of provider: {provider_lookup_key:s}'))
      return None

    message_strings = self._GetMessageStrings(
        storage_reader, message_file_identifiers, message_identifier)
    if not message_strings:
      logger.warning((
          f'No message string for identifier: 0x{message_identifier:08x} '
          f'(original: 0x{original_message_identifier:08x}) '
          f'of provider: {provider_lookup_key:s}'))
      return None

    return message_strings[0].string

  def _ReadParameterMessageString(
      self, storage_reader, provider_identifier, log_source,
      message_identifier):
    """Reads a parameter message string.

    Args:
      storage_reader (StorageReader): storage reader.
      provider_identifier (str): EventLog provider identifier.
      log_source (str): EventLog source, such as "Application Error".
      message_identifier (int): message identifier.

    Returns:
      str: parameter string or None if not available.
    """
    if self._environment_variables is None:
      self._ReadEnvironmentVariables(storage_reader)

    if self._windows_eventlog_providers is None:
      self._ReadWindowsEventLogProviders(storage_reader)

    if self._windows_eventlog_message_files is None:
      self._ReadWindowsEventLogMessageFiles(storage_reader)

    provider, provider_lookup_key = self._GetWindowsEventLogProvider(
        provider_identifier, log_source)
    if not provider:
      return None

    if not storage_reader.HasAttributeContainers(
        'windows_eventlog_message_string'):
      return None

    message_files = provider.parameter_message_files
    if not message_files:
      # If no parameter message files are defined fallback to the event
      # message files and default parameter message files.
      message_files = list(provider.event_message_files)
      message_files.extend(self._DEFAULT_PARAMETER_MESSAGE_FILES)

    message_file_identifiers = self._GetEventMessageFileIdentifiers(
        message_files)
    if not message_file_identifiers:
      logger.warning((
          f'No parameter message file for identifier: '
          f'0x{message_identifier:08x} of provider: {provider_lookup_key:s}'))
      return None

    message_strings = self._GetMessageStrings(
        storage_reader, message_file_identifiers, message_identifier)
    if not message_strings:
      logger.warning((
          f'No parameter string for identifier: 0x{message_identifier:08x} '
          f'of provider: {provider_lookup_key:s}'))
      return None

    return message_strings[0].string

  def _ReadWindowsEventLogMessageFiles(
      self, attribute_store, container_type='windows_eventlog_message_file',
      path_attribute='path'):
    """Reads the Windows EventLog message files.

    Args:
      attribute_store (AttributeContainerStore): attribute container store.
      container_type (Optional[str]): attribute container type.
      path_attribute (Optional[str]): name of the attribute containing the path.
    """
    # TODO: get windows eventlog message files related to the source.
    self._windows_eventlog_message_files = {}
    if attribute_store.HasAttributeContainers(container_type):
      for message_file in attribute_store.GetAttributeContainers(
          container_type):
        message_file_path = getattr(message_file, path_attribute, None)
        path, filename = path_helper.PathHelper.GetWindowsSystemPath(
            message_file_path, self._environment_variables)

        lookup_path = '\\'.join([path, filename]).lower()
        message_file_identifier = message_file.GetIdentifier()
        self._windows_eventlog_message_files[lookup_path] = (
            message_file_identifier)

  def _ReadWindowsEventLogProviders(
      self, attribute_store, container_type='windows_eventlog_provider'):
    """Reads Windows EventLog provider attribute containers.

    Args:
      attribute_store (AttributeContainerStore): attribute container store.
      container_type (Optional[str]): attribute container type.
    """
    self._windows_eventlog_providers = {}
    if attribute_store.HasAttributeContainers(container_type):
      for provider in attribute_store.GetAttributeContainers(container_type):
        if provider.identifier:
          self._windows_eventlog_providers[provider.identifier] = provider

        for log_source in provider.log_sources:
          log_source = log_source.lower()
          self._windows_eventlog_providers[log_source] = provider

  def GetMessageString(
      self, provider_identifier, log_source, message_identifier, event_version):
    """Retrieves a specific Windows EventLog message string.

    Args:
      provider_identifier (str): EventLog provider identifier.
      log_source (str): EventLog source, such as "Application Error".
      message_identifier (int): message identifier.
      event_version (int): event version or None if not set.

    Returns:
      str: message string or None if not available.
    """
    message_string = self._GetCachedMessageString(
        provider_identifier, log_source, message_identifier, event_version)
    if not message_string:
      if self._storage_reader:
        message_string = self._ReadEventMessageString(
            self._storage_reader, provider_identifier, log_source,
            message_identifier, event_version)
      else:
        message_string = self._GetWinevtRcDatabaseMessageString(
            provider_identifier, log_source, message_identifier, event_version)

      if message_string:
        self._CacheMessageString(
            provider_identifier, log_source, message_identifier, event_version,
            message_string)

    return message_string

  def GetParameterString(
      self, provider_identifier, log_source, message_identifier):
    """Retrieves a specific Windows EventLog parameter string.

    Args:
      provider_identifier (str): EventLog provider identifier.
      log_source (str): EventLog source, such as "Application Error".
      message_identifier (int): parameter identifier.

    Returns:
      str: parameter string or None if not available.
    """
    message_string = self._GetCachedMessageString(
        provider_identifier, log_source, message_identifier, None)
    if not message_string:
      message_string = self._ReadParameterMessageString(
          self._storage_reader, provider_identifier, log_source,
          message_identifier)

      if message_string:
        self._CacheMessageString(
            provider_identifier, log_source, message_identifier,
            None, message_string)

    return message_string



# -*- coding: utf-8 -*-
"""Analysis plugin that labels events by session."""

from plaso.analysis import interface
from plaso.analysis import manager
from plaso.lib import definitions


class SessionizeAnalysisPlugin(interface.AnalysisPlugin):
  """Analysis plugin that labels events by session."""

  NAME = 'sessionize'

  _DEFAULT_MAXIMUM_PAUSE = 10 * definitions.MICROSECONDS_PER_MINUTE

  def __init__(self):
    """Initializes a sessionize analysis plugin."""
    super(SessionizeAnalysisPlugin, self).__init__()
    self._current_session_number = 0
    self._maximum_pause_microseconds = self._DEFAULT_MAXIMUM_PAUSE
    self._session_end_timestamp = None

  # pylint: disable=unused-argument
  def ExamineEvent(
      self, analysis_mediator, event, event_data, event_data_stream):
    """Analyzes an EventObject and tags it as part of a session.

    Args:
      analysis_mediator (AnalysisMediator): mediates interactions between
          analysis plugins and other components, such as storage and dfVFS.
      event (EventObject): event to examine.
      event_data (EventData): event data.
      event_data_stream (EventDataStream): event data stream.
    """
    if (self._session_end_timestamp is not None and
        event.timestamp > self._session_end_timestamp):
      self._current_session_number += 1

    self._session_end_timestamp = (
        event.timestamp + self._maximum_pause_microseconds)

    label = f'session_{self._current_session_number:d}'

    event_tag = self._CreateEventTag(event, [label])
    analysis_mediator.ProduceEventTag(event_tag)

    self._analysis_counter[label] += 1

  def SetMaximumPause(self, maximum_pause_minutes):
    """Sets the maximum pause interval between events to consider a session.

    Args:
      maximum_pause_minutes (int): maximum pause interval between events that
          are considered part of the same session, in minutes.
    """
    self._maximum_pause_microseconds = (
        maximum_pause_minutes * definitions.MICROSECONDS_PER_MINUTE)


manager.AnalysisPluginManager.RegisterPlugin(SessionizeAnalysisPlugin)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the sessions attribute containers."""

import unittest

from plaso.containers import sessions

from tests import test_lib as shared_test_lib


class SessionTest(shared_test_lib.BaseTestCase):
  """Tests for the session attribute container."""

  def testGetAttributeNames(self):
    """Tests the GetAttributeNames function."""
    attribute_container = sessions.Session()

    expected_attribute_names = [
        'aborted',
        'artifact_filters',
        'command_line_arguments',
        'completion_time',
        'debug_mode',
        'enabled_parser_names',
        'filter_file',
        'identifier',
        'parser_filter_expression',
        'preferred_codepage',
        'preferred_encoding',
        'preferred_language',
        'preferred_time_zone',
        'preferred_year',
        'product_name',
        'product_version',
        'start_time']

    attribute_names = sorted(attribute_container.GetAttributeNames())

    self.assertEqual(attribute_names, expected_attribute_names)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Windows EventLog providers helper."""

import unittest

from plaso.helpers.windows import eventlog_providers

from tests import test_lib as shared_test_lib


class WindowsEventLogProvidersHelperTest(shared_test_lib.BaseTestCase):
  """Tests for the Windows EventLog providers helper."""

  # pylint: disable=protected-access

  def testGetNormalizedPath(self):
    """Tests the _GetNormalizedPath function."""
    test_helper = eventlog_providers.WindowsEventLogProvidersHelper()

    normalized_path = test_helper._GetNormalizedPath(
        '%SystemRoot%\\System32\\IoLogMsg.dll')
    self.assertEqual(normalized_path, '%SystemRoot%\\System32\\IoLogMsg.dll')

    normalized_path = test_helper._GetNormalizedPath(
        '%windir%\\System32\\lsasrv.dll')
    self.assertEqual(normalized_path, '%SystemRoot%\\System32\\lsasrv.dll')

    normalized_path = test_helper._GetNormalizedPath(
        'C:\\Windows\\System32\\mscoree.dll')
    self.assertEqual(normalized_path, '%SystemRoot%\\System32\\mscoree.dll')

    normalized_path = test_helper._GetNormalizedPath(
        'werfault.exe')
    self.assertEqual(normalized_path, '%SystemRoot%\\System32\\werfault.exe')

    normalized_path = test_helper._GetNormalizedPath(
        'system32\\drivers\\WdFilter.sys')
    self.assertEqual(normalized_path, (
        '%SystemRoot%\\System32\\drivers\\WdFilter.sys'))

    normalized_path = test_helper._GetNormalizedPath(
        'C:\\Program Files (x86)\\Microsoft\\Edge\\Application\\84.0.522.52\\'
        'eventlog_provider.dll')
    self.assertEqual(normalized_path, (
        '\\Program Files (x86)\\Microsoft\\Edge\\Application\\84.0.522.52\\'
        'eventlog_provider.dll'))

    normalized_path = test_helper._GetNormalizedPath(
        '%ProgramFiles%\\Windows Defender\\MpClient.dll')
    self.assertEqual(normalized_path, (
        '%ProgramFiles%\\Windows Defender\\MpClient.dll'))

    normalized_path = test_helper._GetNormalizedPath(
        '%programdata%\\Microsoft\\Windows Defender\\Definition Updates\\'
        'Default\\MpEngine.dll')
    self.assertEqual(normalized_path, (
        '%programdata%\\Microsoft\\Windows Defender\\Definition Updates\\'
        'Default\\MpEngine.dll'))

    normalized_path = test_helper._GetNormalizedPath(
        '$(runtime.system32)\\WinML.dll')
    self.assertEqual(normalized_path, '%SystemRoot%\\System32\\WinML.dll')

    normalized_path = test_helper._GetNormalizedPath(
        '$(runtime.windows)\\immersivecontrolpanel\\systemsettings.exe')
    self.assertEqual(normalized_path, (
        '%SystemRoot%\\immersivecontrolpanel\\systemsettings.exe'))

    normalized_path = test_helper._GetNormalizedPath('\\eventlogmessages.dll')
    self.assertEqual(normalized_path, '\\eventlogmessages.dll')

    # TODO: add tests for Merge
    # TODO: add tests for NormalizeMessageFiles


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Text parser plugins for SkyDrive version 1 and 2 log files."""

import pyparsing

from dfdatetime import time_elements as dfdatetime_time_elements

from plaso.containers import events
from plaso.lib import errors
from plaso.parsers import logger
from plaso.parsers import text_parser
from plaso.parsers.text_plugins import interface


class SkyDriveLogEventData(events.EventData):
  """SkyDrive log event data.

  Attributes:
    added_time (dfdatetime.DateTimeValues): date and time the log entry
        was added.
    detail (str): detail.
    log_level (str): log level.
    module (str): name of the module that generated the log message.
    source_code (str): source file and line number that generated the log
        message.
  """

  DATA_TYPE = 'skydrive:log:entry'

  def __init__(self):
    """Initializes event data."""
    super(SkyDriveLogEventData, self).__init__(data_type=self.DATA_TYPE)
    self.added_time = None
    self.detail = None
    self.log_level = None
    self.module = None
    self.source_code = None


class SkyDriveLog1TextPlugin(interface.TextPlugin):
  """Text parser plugin for SkyDrive version 1 log files."""

  NAME = 'skydrive_log_v1'
  DATA_FORMAT = 'OneDrive (or SkyDrive) version 1 log file'

  ENCODING = 'utf-8'

  _INTEGER = pyparsing.Word(pyparsing.nums).set_parse_action(
      lambda tokens: int(tokens[0], 10))

  _TWO_DIGITS = pyparsing.Word(pyparsing.nums, exact=2).set_parse_action(
      lambda tokens: int(tokens[0], 10))

  _THREE_DIGITS = pyparsing.Word(pyparsing.nums, exact=3).set_parse_action(
      lambda tokens: int(tokens[0], 10))

  _FOUR_DIGITS = pyparsing.Word(pyparsing.nums, exact=4).set_parse_action(
      lambda tokens: int(tokens[0], 10))

  # Format version 1 date and time values are formatted as:
  # DD-MM-YYYY hhmmss.###
  # For example: 08-01-2013 21:22:28.999
  _DATE_TIME_V1 = pyparsing.Group(
      _TWO_DIGITS + pyparsing.Suppress('-') +
      _TWO_DIGITS + pyparsing.Suppress('-') + _FOUR_DIGITS +
      _TWO_DIGITS + pyparsing.Suppress(':') +
      _TWO_DIGITS + pyparsing.Suppress(':') + _TWO_DIGITS +
      pyparsing.Word('.,', exact=1).suppress() +
                     _THREE_DIGITS).set_results_name('date_time')

  _SOURCE_CODE = pyparsing.Combine(
      pyparsing.CharsNotIn(':') + pyparsing.Literal(':') + _INTEGER +
      pyparsing.Literal('!') +
      pyparsing.Word(pyparsing.printables)).set_results_name('source_code')

  _END_OF_LINE = pyparsing.Suppress(pyparsing.LineEnd())

  _LOG_LINE_V1 = (
      _DATE_TIME_V1 + _SOURCE_CODE +
      pyparsing.QuotedString('(', endQuoteChar=')').set_results_name(
          'log_level') + pyparsing.Suppress(':') +
      pyparsing.restOfLine().set_results_name('detail') + _END_OF_LINE)

  # Sometimes the timestamped log line is followed by an empty line,
  # then by a file name plus other data and finally by another empty
  # line. It could happen that a log line is split in two parts.
  # These lines will not be discarded and an event will be generated
  # ad-hoc (see source), based on the last one if available.
  _NO_HEADER_SINGLE_LINE = (
      pyparsing.NotAny(_DATE_TIME_V1) +
      pyparsing.Optional(pyparsing.Suppress('->')) +
      pyparsing.restOfLine().set_results_name('detail') +
      _END_OF_LINE)

  # Define the available log line structures.
  _LINE_STRUCTURES = [
      ('log_line_v1', _LOG_LINE_V1),
      ('no_header_single_line', _NO_HEADER_SINGLE_LINE)]

  VERIFICATION_GRAMMAR = _LOG_LINE_V1

  def __init__(self):
    """Initializes a text parser plugin."""
    super(SkyDriveLog1TextPlugin, self).__init__()
    self._event_data = None

  def _ParseLoglineVersion1(self, parser_mediator, structure):
    """Parse a version 1 log line.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      structure (pyparsing.ParseResults): structure of tokens derived from
          a line of a text file.
    """
    time_elements_structure = self._GetValueFromStructure(
        structure, 'date_time')

    event_data = SkyDriveLogEventData()
    event_data.added_time = self._ParseTimeElementsVersion1(
        time_elements_structure)
    event_data.detail = self._GetStringValueFromStructure(structure, 'detail')
    event_data.log_level = self._GetValueFromStructure(structure, 'log_level')
    event_data.source_code = self._GetValueFromStructure(
        structure, 'source_code')

    parser_mediator.ProduceEventData(event_data)

    self._event_data = event_data

  def _ParseNoHeaderSingleLine(self, parser_mediator, structure):
    """Parse an isolated header line and store appropriate attributes.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      structure (pyparsing.ParseResults): structure of tokens derived from
          a line of a text file.
    """
    if not self._event_data:
      logger.debug('SkyDrive, found isolated line with no previous events')
      return

    event_data = SkyDriveLogEventData()
    event_data.added_time = self._event_data.added_time
    event_data.detail = self._GetValueFromStructure(structure, 'detail')

    parser_mediator.ProduceEventData(event_data)

    self._ResetState()

  def _ParseRecord(self, parser_mediator, key, structure):
    """Parses a pyparsing structure.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      key (str): name of the parsed structure.
      structure (pyparsing.ParseResults): tokens from a parsed log line.

    Raises:
      ParseError: if the structure cannot be parsed.
    """
    if key == 'log_line_v1':
      self._ParseLoglineVersion1(parser_mediator, structure)

    elif key == 'no_header_single_line':
      self._ParseNoHeaderSingleLine(parser_mediator, structure)

  def _ParseTimeElementsVersion1(self, time_elements_structure):
    """Parses date and time elements of a version 1 log line.

    Args:
      time_elements_structure (pyparsing.ParseResults): date and time elements
          of a log line.

    Returns:
      dfdatetime.TimeElements: date and time value.

    Raises:
      ParseError: if a valid date and time value cannot be derived from
          the time elements.
    """
    try:
      # Ensure time_elements_tuple is not a pyparsing.ParseResults otherwise
      # copy.deepcopy() of the dfDateTime object will fail on Python 3.8 with:
      # "TypeError: 'str' object is not callable" due to pyparsing.ParseResults
      # overriding __getattr__ with a function that returns an empty string
      # when named token does not exists.

      month, day_of_month, year, hours, minutes, seconds, milliseconds = (
          time_elements_structure)

      time_elements_tuple = (
          year, month, day_of_month, hours, minutes, seconds, milliseconds)

      # TODO: determine if this should be local time.
      return dfdatetime_time_elements.TimeElementsInMilliseconds(
          time_elements_tuple=time_elements_tuple)

    except (TypeError, ValueError) as exception:
      raise errors.ParseError(
          'Unable to parse time elements with error: {0!s}'.format(exception))

  def _ResetState(self):
    """Resets stored values."""
    self._event_data = None

  def CheckRequiredFormat(self, parser_mediator, text_reader):
    """Check if the log record has the minimal structure required by the plugin.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      text_reader (EncodedTextReader): text reader.

    Returns:
      bool: True if this is the correct plugin, False otherwise.
    """
    try:
      structure = self._VerifyString(text_reader.lines)
    except errors.ParseError:
      return False

    time_elements_structure = self._GetValueFromStructure(
        structure, 'date_time')

    try:
      self._ParseTimeElementsVersion1(time_elements_structure)
    except errors.ParseError:
      return False

    self._ResetState()

    return True


class SkyDriveLog2TextPlugin(interface.TextPluginWithLineContinuation):
  """Text parser plugin for SkyDrive version 2 log files."""

  NAME = 'skydrive_log_v2'
  DATA_FORMAT = 'OneDrive (or SkyDrive) version 2 log file'

  ENCODING = 'utf-8'

  _TWO_DIGITS = pyparsing.Word(pyparsing.nums, exact=2).set_parse_action(
      lambda tokens: int(tokens[0], 10))

  _THREE_DIGITS = pyparsing.Word(pyparsing.nums, exact=3).set_parse_action(
      lambda tokens: int(tokens[0], 10))

  _FOUR_DIGITS = pyparsing.Word(pyparsing.nums, exact=4).set_parse_action(
      lambda tokens: int(tokens[0], 10))

  # Format version 2 header date and time values are formatted as:
  # YYYY-MM-DD-hhmmss.###
  # For example: 2013-07-25-160323.291
  _HEADER_DATE_TIME_V2 = pyparsing.Group(
      _FOUR_DIGITS + pyparsing.Suppress('-') +
      _TWO_DIGITS + pyparsing.Suppress('-') +
      _TWO_DIGITS + pyparsing.Suppress('-') +
      _TWO_DIGITS + _TWO_DIGITS + _TWO_DIGITS +
      pyparsing.Suppress('.') + _THREE_DIGITS).set_results_name(
          'header_date_time')

  # Formate version 2 date and time values are formatted as:
  # MM-DD-YY,hh:mm:ss.###
  # For example: 07-25-13,16:06:31.820
  _DATE_TIME_V2 = (
      _TWO_DIGITS + pyparsing.Suppress('-') +
      _TWO_DIGITS + pyparsing.Suppress('-') +
      _TWO_DIGITS + pyparsing.Suppress(',') +
      _TWO_DIGITS + pyparsing.Suppress(':') +
      _TWO_DIGITS + pyparsing.Suppress(':') + _TWO_DIGITS +
      pyparsing.Suppress('.') + _THREE_DIGITS).set_results_name('date_time')

  _END_OF_LINE = pyparsing.Suppress(pyparsing.LineEnd())

  _HEADER_LINE_V2_START = (
      pyparsing.Suppress('######') +
      pyparsing.Literal('Logging started.').set_results_name('log_start') +
      pyparsing.Literal('Version=').set_results_name('version_string') +
      pyparsing.Word(pyparsing.nums + '.').set_results_name('version_number') +
      pyparsing.Suppress('StartSystemTime:') + _HEADER_DATE_TIME_V2 +
      pyparsing.Literal('StartLocalTime:').set_results_name(
          'local_time_string'))

  _HEADER_LINE_V2 = (
      _HEADER_LINE_V2_START + pyparsing.restOfLine.set_results_name('detail') +
      _END_OF_LINE)

  _LOG_LINE_V2_START = (
      _DATE_TIME_V2 + pyparsing.Suppress(',') +
      pyparsing.Word(pyparsing.hexnums) + pyparsing.Suppress(',') +
      pyparsing.Word(pyparsing.hexnums) + pyparsing.Suppress(',') +
      pyparsing.Word(pyparsing.hexnums) + pyparsing.Suppress(',') +
      pyparsing.CharsNotIn(',').set_results_name('module') +
      pyparsing.Suppress(',') +
      pyparsing.CharsNotIn(',').set_results_name('source_code') +
      pyparsing.Suppress(',') +
      pyparsing.Word(pyparsing.hexnums) + pyparsing.Suppress(',') +
      pyparsing.Word(pyparsing.hexnums) + pyparsing.Suppress(',') +
      pyparsing.CharsNotIn(',').set_results_name('log_level') +
      pyparsing.Suppress(','))

  _LOG_LINE_V2 = (
      _LOG_LINE_V2_START + pyparsing.restOfLine.set_results_name('detail') +
      _END_OF_LINE)

  _LINE_STRUCTURES = [
      ('header_line_v2', _HEADER_LINE_V2),
      ('log_line_v2', _LOG_LINE_V2)]

  VERIFICATION_GRAMMAR = _HEADER_LINE_V2

  def __init__(self):
    """Initializes a text parser plugin."""
    super(SkyDriveLog2TextPlugin, self).__init__()
    self._detail_lines = None
    self._event_data = None

  def _ParseFinalize(self, parser_mediator):
    """Finalizes parsing.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
    """
    if self._event_data:
      self._event_data.detail = ' '.join(self._detail_lines)
      self._detail_lines = None

      parser_mediator.ProduceEventData(self._event_data)
      self._event_data = None

  def _ParseHeaderLine(self, parser_mediator, structure):
    """Parse a header line.

    ['Logging started.', 'Version=', '17.0.2011.0627',
    [2013, 7, 25], 16, 3, 23, 291, 'StartLocalTime', '<detail>']

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      structure (pyparsing.ParseResults): structure of tokens derived from
          a line of a text file.
    """
    time_elements_structure = self._GetValueFromStructure(
        structure, 'header_date_time')

    detail = self._GetValueFromStructure(structure, 'detail')
    local_time_string = self._GetValueFromStructure(
        structure, 'local_time_string')
    log_start = self._GetValueFromStructure(structure, 'log_start')
    version_number = self._GetValueFromStructure(structure, 'version_number')
    version_string = self._GetValueFromStructure(structure, 'version_string')

    event_data = SkyDriveLogEventData()
    event_data.added_time = self._ParseHeaderTimeElements(
        time_elements_structure)
    # TODO: refactor detail to individual event data attributes.
    event_data.detail = '{0!s} {1!s} {2!s} {3!s} {4!s}'.format(
        log_start, version_string, version_number, local_time_string, detail)

    parser_mediator.ProduceEventData(event_data)

  def _ParseHeaderTimeElements(self, time_elements_structure):
    """Parses date and time elements of a header line.

    Args:
      time_elements_structure (pyparsing.ParseResults): date and time elements
          of a log line.

    Returns:
      dfdatetime.TimeElements: date and time value.

    Raises:
      ParseError: if a valid date and time value cannot be derived from
          the time elements.
    """
    try:
      # Ensure time_elements_tuple is not a pyparsing.ParseResults otherwise
      # copy.deepcopy() of the dfDateTime object will fail on Python 3.8 with:
      # "TypeError: 'str' object is not callable" due to pyparsing.ParseResults
      # overriding __getattr__ with a function that returns an empty string
      # when named token does not exists.

      year, month, day_of_month, hours, minutes, seconds, milliseconds = (
          time_elements_structure)

      time_elements_tuple = (
          year, month, day_of_month, hours, minutes, seconds, milliseconds)

      return dfdatetime_time_elements.TimeElementsInMilliseconds(
          time_elements_tuple=time_elements_tuple)

    except (TypeError, ValueError) as exception:
      raise errors.ParseError(
          'Unable to parse time elements with error: {0!s}'.format(exception))

  def _ParseLoglineVersion2(self, structure):
    """Parse a version 2 log line.

    Args:
      structure (pyparsing.ParseResults): structure of tokens derived from
          a line of a text file.
    """
    time_elements_structure = self._GetValueFromStructure(
        structure, 'date_time')

    detail = self._GetValueFromStructure(structure, 'detail', default_value='')
    detail = detail.strip()

    event_data = SkyDriveLogEventData()
    event_data.added_time = self._ParseTimeElementsVersion2(
        time_elements_structure)
    event_data.log_level = self._GetValueFromStructure(structure, 'log_level')
    event_data.module = self._GetValueFromStructure(structure, 'module')
    event_data.source_code = self._GetValueFromStructure(
        structure, 'source_code')

    self._event_data = event_data
    self._detail_lines = [detail]

  def _ParseRecord(self, parser_mediator, key, structure):
    """Parses a pyparsing structure.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      key (str): name of the parsed structure.
      structure (pyparsing.ParseResults): tokens from a parsed log line.

    Raises:
      ParseError: if the structure cannot be parsed.
    """
    if self._event_data and key in ('header_line_v2', 'log_line_v2'):
      self._event_data.detail = ' '.join(self._detail_lines)

      parser_mediator.ProduceEventData(self._event_data)
      self._event_data = None

    if key == 'header_line_v2':
      self._ParseHeaderLine(parser_mediator, structure)

    elif key == '_line_continuation':
      detail = structure.replace('\n', ' ').strip()
      self._detail_lines.append(detail)

    else:
      self._ParseLoglineVersion2(structure)

  def _ParseTimeElementsVersion2(self, time_elements_structure):
    """Parses date and time elements of a version 2 log line.

    Args:
      time_elements_structure (pyparsing.ParseResults): date and time elements
          of a log line.

    Returns:
      dfdatetime.TimeElements: date and time value.

    Raises:
      ParseError: if a valid date and time value cannot be derived from
          the time elements.
    """
    try:
      # Ensure time_elements_tuple is not a pyparsing.ParseResults otherwise
      # copy.deepcopy() of the dfDateTime object will fail on Python 3.8 with:
      # "TypeError: 'str' object is not callable" due to pyparsing.ParseResults
      # overriding __getattr__ with a function that returns an empty string
      # when named token does not exists.

      month, day_of_month, year, hours, minutes, seconds, milliseconds = (
          time_elements_structure)

      year += 2000

      time_elements_tuple = (
          year, month, day_of_month, hours, minutes, seconds, milliseconds)

      return dfdatetime_time_elements.TimeElementsInMilliseconds(
          time_elements_tuple=time_elements_tuple)

    except (TypeError, ValueError) as exception:
      raise errors.ParseError(
          'Unable to parse time elements with error: {0!s}'.format(exception))

  def _ResetState(self):
    """Resets stored values."""
    self._detail_lines = []
    self._event_data = None

  def CheckRequiredFormat(self, parser_mediator, text_reader):
    """Check if the log record has the minimal structure required by the parser.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      text_reader (EncodedTextReader): text reader.

    Returns:
      bool: True if this is the correct plugin, False otherwise.
    """
    try:
      structure = self._VerifyString(text_reader.lines)
    except errors.ParseError:
      return False

    time_elements_tuple = self._GetValueFromStructure(
        structure, 'header_date_time')

    try:
      dfdatetime_time_elements.TimeElementsInMilliseconds(
          time_elements_tuple=time_elements_tuple)
    except ValueError:
      return False

    self._ResetState()

    return True


text_parser.TextLogParser.RegisterPlugins([
    SkyDriveLog1TextPlugin, SkyDriveLog2TextPlugin])



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Mozilla Firefox history custom event formatter helpers."""

import unittest

from plaso.formatters import firefox

from tests.formatters import test_lib


class FirefoxHistoryTypedCountFormatterHelperTest(
    test_lib.EventFormatterTestCase):
  """Tests for the Mozilla Firefox history typed count formatter helper."""

  def testFormatEventValues(self):
    """Tests the FormatEventValues function."""
    formatter_helper = firefox.FirefoxHistoryTypedCountFormatterHelper()

    output_mediator = self._CreateOutputMediator()

    event_values = {'typed': '1'}
    formatter_helper.FormatEventValues(output_mediator, event_values)
    self.assertEqual(event_values['url_typed_string'], '(URL directly typed)')

    event_values = {'typed': None}
    formatter_helper.FormatEventValues(output_mediator, event_values)
    self.assertEqual(
        event_values['url_typed_string'], '(URL not typed directly)')


class FirefoxHistoryURLHiddenFormatterHelperTest(
    test_lib.EventFormatterTestCase):
  """Tests for the Mozilla Firefox history URL hidden formatter helper."""

  def testFormatEventValues(self):
    """Tests the FormatEventValues function."""
    formatter_helper = firefox.FirefoxHistoryURLHiddenFormatterHelper()

    output_mediator = self._CreateOutputMediator()

    event_values = {'hidden': '1'}
    formatter_helper.FormatEventValues(output_mediator, event_values)
    self.assertEqual(event_values['url_hidden_string'], '(URL hidden)')

    event_values = {'hidden': None}
    formatter_helper.FormatEventValues(output_mediator, event_values)
    self.assertNotIn('url_hidden_string', event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the CLI tool options mix-ins."""

import argparse
import unittest

from plaso.cli import tool_options
from plaso.cli import tools
from plaso.output import manager as output_manager

from tests.cli import test_lib


class TestToolWithAnalysisPluginOptions(
    tools.CLITool, tool_options.AnalysisPluginOptions):
  """Tool to test the analysis plugin options."""

  def __init__(self, input_reader=None, output_writer=None):
    """Initializes the CLI tool object.

    Args:
      input_reader (Optional[InputReader]): input reader, where None indicates
          that the stdin input reader should be used.
      output_writer (Optional[OutputWriter]): output writer, where None
          indicates that the stdout output writer should be used.
    """
    super(TestToolWithAnalysisPluginOptions, self).__init__(
        input_reader=input_reader, output_writer=output_writer)
    self._analysis_plugins = None


class AnalysisPluginOptionsTest(test_lib.CLIToolTestCase):
  """Tests for the analysis plugin options."""

  # pylint: disable=protected-access

  def testCreateAnalysisPlugins(self):
    """Tests the _CreateAnalysisPlugins function."""
    test_file_path = self._GetTestFilePath(['tagging_file', 'valid.txt'])
    self._SkipIfPathNotExists(test_file_path)

    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = TestToolWithAnalysisPluginOptions(output_writer=output_writer)

    options = test_lib.TestOptions()
    options.tagging_file = test_file_path

    test_tool._analysis_plugins = 'tagging'
    plugins = test_tool._CreateAnalysisPlugins(options)
    self.assertIn('tagging', plugins.keys())

    test_tool._analysis_plugins = 'bogus'
    plugins = test_tool._CreateAnalysisPlugins(options)
    self.assertEqual(plugins, {})

    test_tool._analysis_plugins = ''
    plugins = test_tool._CreateAnalysisPlugins(options)
    self.assertEqual(plugins, {})

  def testListAnalysisPlugins(self):
    """Tests the ListAnalysisPlugins function."""
    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = TestToolWithAnalysisPluginOptions(output_writer=output_writer)

    test_tool.ListAnalysisPlugins()

    output = output_writer.ReadOutput()

    number_of_tables = 0
    lines = []
    for line in output.split('\n'):
      line = line.strip()
      lines.append(line)

      if line.startswith('*****') and line.endswith('*****'):
        number_of_tables += 1

    self.assertIn('Analysis Plugins', lines[1])

    lines = frozenset(lines)

    self.assertEqual(number_of_tables, 1)

    expected_line = (
        'browser_search : Analyze browser search entries from events.')
    self.assertIn(expected_line, lines)


class TestToolWithHashersOptions(
    tools.CLITool, tool_options.HashersOptions):
  """Tool to test the hashers options."""


class HashersOptionsTest(test_lib.CLIToolTestCase):
  """Tests for the hashers options."""

  def testListHashers(self):
    """Tests the ListHashers function."""
    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = TestToolWithHashersOptions(output_writer=output_writer)

    test_tool.ListHashers()

    output = output_writer.ReadOutput()

    number_of_tables = 0
    lines = []
    for line in output.split('\n'):
      line = line.strip()
      lines.append(line)

      if line.startswith('*****') and line.endswith('*****'):
        number_of_tables += 1

    self.assertIn('Hashers', lines[1])

    lines = frozenset(lines)

    self.assertEqual(number_of_tables, 1)

    expected_line = 'md5 : Calculates an MD5 digest hash over input data.'
    self.assertIn(expected_line, lines)


class TestToolWithOutputModuleOptions(
    tools.CLITool, tool_options.OutputModuleOptions):
  """Tool to test the output module options."""


class OutputModuleOptionsTest(test_lib.CLIToolTestCase):
  """Tests for the output module options."""

  # pylint: disable=protected-access

  _EXPECTED_OUTPUT_TIME_ZONE_OPTION = """\
usage: tool_options.py [--additional_fields ADDITIONAL_FIELDS]
                       [--custom_fields CUSTOM_FIELDS]
                       [--custom_formatter_definitions PATH] [--dynamic_time]
                       [--output_time_zone TIME_ZONE]

Test argument parser.

{0:s}:
  --additional_fields ADDITIONAL_FIELDS, --additional-fields ADDITIONAL_FIELDS
                        Defines additional fields to be included in the output
                        besides the default fields. Multiple additional field
                        names can be defined as a list of comma separated
                        values. Output formats that support additional fields
                        are: dynamic, opensearch and xlsx.
  --custom_fields CUSTOM_FIELDS, --custom-fields CUSTOM_FIELDS
                        Defines custom fields to be included in the output
                        besides the default fields. A custom field is defined
                        as "name:value". Multiple custom field names can be
                        defined as list of comma separated values. Note that
                        regular fields will are favoured above custom fields
                        with same name. Output formats that support this are:
                        dynamic, opensearch and xlsx.
  --custom_formatter_definitions PATH, --custom-formatter-definitions PATH
                        Path to a file containing custom event formatter
                        definitions, which is a .yaml file. Custom event
                        formatter definitions can be used to customize event
                        messages and override the built-in event formatter
                        definitions.
  --dynamic_time, --dynamic-time
                        Indicate that the output should use dynamic time.
                        Output formats that support dynamic time are: dynamic
  --output_time_zone TIME_ZONE, --output-time-zone TIME_ZONE
                        time zone of date and time values written to the
                        output, if supported by the output format. Use "list"
                        to see a list of available time zones. Output formats
                        that support an output time zone are: dynamic and
                        l2t_csv.
""".format(test_lib.ARGPARSE_OPTIONS)

  def testGetOutputModulesInformation(self):
    """Tests the _GetOutputModulesInformation function."""
    test_tool = TestToolWithOutputModuleOptions()
    modules_info = test_tool._GetOutputModulesInformation()

    self.assertIsNotNone(modules_info)

    available_module_names = [name for name, _ in modules_info]
    self.assertIn('dynamic', available_module_names)
    self.assertIn('json', available_module_names)

  def testParseOutputOptions(self):
    """Tests the _ParseOutputOptions function."""
    test_tool = TestToolWithOutputModuleOptions()

    options = test_lib.TestOptions()

    test_tool._ParseOutputOptions(options)
    self.assertFalse(test_tool._output_dynamic_time)
    self.assertIsNone(test_tool._output_time_zone)

    options.output_time_zone = 'list'
    test_tool._ParseOutputOptions(options)
    self.assertFalse(test_tool._output_dynamic_time)
    self.assertIsNone(test_tool._output_time_zone)

    options.output_time_zone = 'CET'
    test_tool._ParseOutputOptions(options)
    self.assertFalse(test_tool._output_dynamic_time)
    self.assertEqual(test_tool._output_time_zone, 'CET')

  def testAddOutputOptions(self):
    """Tests the AddOutputOptions function."""
    argument_parser = argparse.ArgumentParser(
        prog='tool_options.py', description='Test argument parser.',
        add_help=False, formatter_class=test_lib.SortedArgumentsHelpFormatter)

    test_tool = TestToolWithOutputModuleOptions()
    test_tool.AddOutputOptions(argument_parser)

    output = self._RunArgparseFormatHelp(argument_parser)
    self.assertEqual(output, self._EXPECTED_OUTPUT_TIME_ZONE_OPTION)

  def testListOutputModules(self):
    """Tests the ListOutputModules function."""
    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = TestToolWithOutputModuleOptions(output_writer=output_writer)

    test_tool.ListOutputModules()

    output = output_writer.ReadOutput()

    number_of_tables = 0
    lines = []
    for line in output.split('\n'):
      line = line.strip()
      lines.append(line)

      if line.startswith('*****') and line.endswith('*****'):
        number_of_tables += 1

    self.assertIn('Output Modules', lines[1])

    lines = frozenset(lines)
    disabled_outputs = list(
        output_manager.OutputManager.GetDisabledOutputClasses())
    enabled_outputs = list(output_manager.OutputManager.GetOutputClasses())

    expected_number_of_tables = 0
    if disabled_outputs:
      expected_number_of_tables += 1
    if enabled_outputs:
      expected_number_of_tables += 1

    self.assertEqual(number_of_tables, expected_number_of_tables)
    expected_line = 'rawpy : native (or "raw") Python output.'
    self.assertIn(expected_line, lines)


class TestToolWithProfilingOptions(
    tools.CLITool, tool_options.ProfilingOptions):
  """Tool to test the profiling options."""


class ProfilingOptionsTest(test_lib.CLIToolTestCase):
  """Tests for the profiling options."""

  # pylint: disable=protected-access

  _EXPECTED_OUTPUT = """\

********************************** Profilers ***********************************
         Name : Description
--------------------------------------------------------------------------------
    analyzers : Profile CPU time of analyzers, like hashing
format_checks : Profile CPU time per format check
       memory : Profile memory usage over time
      parsers : Profile CPU time per parser
   processing : Profile CPU time of processing phases
  serializers : Profile CPU time of serialization
      storage : Profile storage reads and writes
   task_queue : Profile task queue status (multi-processing only)
        tasks : Profile the status of tasks (multi-processing only)
--------------------------------------------------------------------------------
"""

  def testListProfilers(self):
    """Tests the ListProfilers function."""
    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = TestToolWithProfilingOptions(output_writer=output_writer)

    test_tool.ListProfilers()

    output = output_writer.ReadOutput()
    self.assertEqual(output, self._EXPECTED_OUTPUT)


class TestToolWithStorageFileOptions(
    tools.CLITool, tool_options.StorageFileOptions):
  """Tool to test the storage file options."""


class StorageFileOptionsTest(test_lib.CLIToolTestCase):
  """Tests for the storage file options."""

  # TODO: add test for _CheckStorageFile


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the SkyDrive version 1 and 2 log files text parser plugins."""

import unittest

from plaso.parsers.text_plugins import skydrivelog

from tests.parsers.text_plugins import test_lib


class SkyDriveLog1TextPluginTest(test_lib.TextPluginTestCase):
  """Tests for the SkyDrive version 1 log files text parser plugin."""

  def testProcess(self):
    """Tests the Process function."""
    plugin = skydrivelog.SkyDriveLog1TextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(['skydrive_v1.log'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 18)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'added_time': '2013-08-01T21:22:28.999+00:00',
        'data_type': 'skydrive:log:entry',
        'detail': '17.0.2011.0627 (Ship)',
        'log_level': 'DETAIL',
        'module': None,
        'source_code': 'global.cpp:626!logVersionInfo'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


class SkyDriveLog2TextPluginTest(test_lib.TextPluginTestCase):
  """Tests for the SkyDrive version 2 log files text parser plugin."""

  def testProcess(self):
    """Tests the Process function."""
    plugin = skydrivelog.SkyDriveLog2TextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(['skydrive.log'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 17)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'added_time': '2013-08-12T02:52:32.976+00:00',
        'data_type': 'skydrive:log:entry',
        'detail': (
            'Received data from server,dwID=0x0;dwSize=0x15a;pbData=GET 5 '
            'WNS 331 Context: 2891  <channel-response><id>1;'
            '13714367258539257282</id><exp>2013-09-11T02:52:37Z</exp><url>'
            'https://bn1.notify.windows.com/?token=AgYAAAAdkHjSxiNH1mbF0Rp'
            '5TIv0Kz317BKYIAfBNO6szULCOEE2393owBINnPC5xoika5SJlNtXZ%2bwzaR'
            'VsPRcP1p64XFn90vGwr07DGZxfna%2bxBpBBplzZhLV9y%2fNV%2bBPxNmTI5'
            'sRgaZ%2foGvYCIj6MdeU1</url></channel-response>'),
        'log_level': 'VRB',
        'module': 'WNS',
        'source_code': 'absconn.cpp(177)'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 11)
    self.CheckEventData(event_data, expected_event_values)

  def testProcessWithErrorLog(self):
    """Tests the Process function with a error log."""
    plugin = skydrivelog.SkyDriveLog2TextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(['skydriveerr.log'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 19)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    # Check parsing of a header line.
    expected_event_values = {
        'added_time': '2013-07-25T16:03:23.291+00:00',
        'data_type': 'skydrive:log:entry',
        'detail': (
            'Logging started. Version= 17.0.2011.0627 StartLocalTime: '
            '2013-07-25-180323.291 PID=0x8f4 TID=0x718 ContinuedFrom=')}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

    # Check parsing of a log line.
    expected_event_values = {
        'added_time': '2013-07-25T16:03:24.649+00:00',
        'data_type': 'skydrive:log:entry',
        'detail': 'Sign in failed : DRX_E_AUTH_NO_VALID_CREDENTIALS,',
        'log_level': 'ERR',
        'module': 'AUTH',
        'source_code': 'authapi.cpp(280)'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 1)
    self.CheckEventData(event_data, expected_event_values)

  def testProcessWithUnicodeErrorLog(self):
    """Tests the Process function with an Unicode error log."""
    plugin = skydrivelog.SkyDriveLog2TextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(
        ['skydriveerr-unicode.log'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 19)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'added_time': '2013-07-25T16:04:02.669+00:00',
        'data_type': 'skydrive:log:entry',
        'detail': (
            'No node found named Passport-Jméno-člena, no user name '
            'available,')}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 3)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Task Scheduler Windows Registry plugin."""

import unittest

from plaso.parsers.winreg_plugins import task_scheduler

from tests.parsers.winreg_plugins import test_lib


class TaskCacheWindowsRegistryPluginTest(test_lib.RegistryPluginTestCase):
  """Tests for the Task Cache key Windows Registry plugin."""

  def testFilters(self):
    """Tests the FILTERS class attribute."""
    plugin = task_scheduler.TaskCacheWindowsRegistryPlugin()

    self._AssertFiltersOnKeyPath(plugin, 'HKEY_LOCAL_MACHINE\\Software', (
        'Microsoft\\Windows NT\\CurrentVersion\\Schedule\\TaskCache'))

    self._AssertNotFiltersOnKeyPath(
        plugin, 'HKEY_LOCAL_MACHINE\\Software', 'Bogus')

  def testProcess(self):
    """Tests the Process function."""
    test_file_entry = self._GetTestFileEntry(['SOFTWARE-RunTests'])
    key_path = (
        'HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows NT\\'
        'CurrentVersion\\Schedule\\TaskCache')

    win_registry = self._GetWinRegistryFromFileEntry(test_file_entry)
    registry_key = win_registry.GetKeyByPath(key_path)

    plugin = task_scheduler.TaskCacheWindowsRegistryPlugin()
    storage_writer = self._ParseKeyWithPlugin(
        registry_key, plugin, file_entry=test_file_entry)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 85)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'task_scheduler:task_cache:entry',
        'key_path': key_path,
        'last_registered_time': '2009-07-14T05:08:50.8116269+00:00',
        'last_written_time': '2009-07-14T04:53:25.8116181+00:00',
        'launch_time': None,
        'task_name': 'SynchronizeTime',
        'task_identifier': '{044A6734-E90E-4F8F-B357-B2DC8AB3B5EC}',
        'unknown_time': None}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the syslog text parser plugin."""

import unittest

from dfvfs.helpers import fake_file_system_builder

from plaso.parsers import text_parser
from plaso.parsers.text_plugins import syslog

from tests.parsers.text_plugins import test_lib


class SyslogTextPluginTest(test_lib.TextPluginTestCase):
  """Tests for the syslog text parser plugin."""

  def testCheckRequiredFormat(self):
    """Tests for the CheckRequiredFormat method."""
    plugin = syslog.SyslogTextPlugin()

    # Check ChromeOS syslog format.
    file_system_builder = fake_file_system_builder.FakeFileSystemBuilder()
    file_system_builder.AddFile('/file.txt', (
        b'2016-10-25T12:37:23.297265-07:00 INFO periodic_scheduler[13707]: '
        b'cleanup_logs: job completed\n'))

    file_entry = file_system_builder.file_system.GetFileEntryByPath('/file.txt')

    parser_mediator = self._CreateParserMediator(None, file_entry=file_entry)

    file_object = file_entry.GetFileObject()
    text_reader = text_parser.EncodedTextReader(file_object)
    text_reader.ReadLines()

    result = plugin.CheckRequiredFormat(parser_mediator, text_reader)
    self.assertTrue(result)

    # Check rsyslog format.
    file_system_builder = fake_file_system_builder.FakeFileSystemBuilder()
    file_system_builder.AddFile('/file.txt', (
        b'2020-05-31T00:00:45.738158+00:00 localhost systemd[1]: Reloaded '
        b'System Logging Service.\n'))

    file_entry = file_system_builder.file_system.GetFileEntryByPath('/file.txt')

    parser_mediator = self._CreateParserMediator(None, file_entry=file_entry)

    file_object = file_entry.GetFileObject()
    text_reader = text_parser.EncodedTextReader(file_object)
    text_reader.ReadLines()

    result = plugin.CheckRequiredFormat(parser_mediator, text_reader)
    self.assertTrue(result)

    # Check protocol 23 rsyslog format.
    file_system_builder = fake_file_system_builder.FakeFileSystemBuilder()
    file_system_builder.AddFile('/file.txt', (
        b'<30>1 2021-03-06T04:07:38.265422+00:00 hostname systemd 1 - -  '
        b'Started Regular background program processing daemon.\n'))

    file_entry = file_system_builder.file_system.GetFileEntryByPath('/file.txt')

    parser_mediator = self._CreateParserMediator(None, file_entry=file_entry)

    file_object = file_entry.GetFileObject()
    text_reader = text_parser.EncodedTextReader(file_object)
    text_reader.ReadLines()

    result = plugin.CheckRequiredFormat(parser_mediator, text_reader)
    self.assertTrue(result)

    # Check traditional syslog format.
    file_system_builder = fake_file_system_builder.FakeFileSystemBuilder()
    file_system_builder.AddFile('/file.txt', (
        b'Jan 22 07:52:33 myhostname.myhost.com client[30840]: INFO No new '
        b'content in \xc3\xadmynd.dd.\n'))

    file_entry = file_system_builder.file_system.GetFileEntryByPath('/file.txt')

    parser_mediator = self._CreateParserMediator(None, file_entry=file_entry)

    file_object = file_entry.GetFileObject()
    text_reader = text_parser.EncodedTextReader(file_object)
    text_reader.ReadLines()

    result = plugin.CheckRequiredFormat(parser_mediator, text_reader)
    self.assertFalse(result)

    # Check traditional rsyslog format.
    file_system_builder = fake_file_system_builder.FakeFileSystemBuilder()
    file_system_builder.AddFile('/file.txt', (
        b'Jan 22 07:54:32 myhostname.myhost.com Job `cron.daily\' '
        b'terminated\n'))

    file_entry = file_system_builder.file_system.GetFileEntryByPath('/file.txt')

    parser_mediator = self._CreateParserMediator(None, file_entry=file_entry)

    file_object = file_entry.GetFileObject()
    text_reader = text_parser.EncodedTextReader(file_object)
    text_reader.ReadLines()

    result = plugin.CheckRequiredFormat(parser_mediator, text_reader)
    self.assertFalse(result)

    # Check syslogkd rsyslog format.
    file_system_builder = fake_file_system_builder.FakeFileSystemBuilder()
    file_system_builder.AddFile('/file.txt', (
        b'Mar  6 04:07:28 hostname systemd[1]: Started Regular background '
        b'program processing daemon.\n'))

    file_entry = file_system_builder.file_system.GetFileEntryByPath('/file.txt')

    parser_mediator = self._CreateParserMediator(None, file_entry=file_entry)

    file_object = file_entry.GetFileObject()
    text_reader = text_parser.EncodedTextReader(file_object)
    text_reader.ReadLines()

    result = plugin.CheckRequiredFormat(parser_mediator, text_reader)
    self.assertFalse(result)

    # Check non-syslog format.
    file_system_builder = fake_file_system_builder.FakeFileSystemBuilder()
    file_system_builder.AddFile('/file.txt', (
        b'gpgv: Signature made Wed Oct 22 17:40:30 2014 UTC using DSA key ID '
        b'437D05B5\n'))

    file_entry = file_system_builder.file_system.GetFileEntryByPath('/file.txt')

    parser_mediator = self._CreateParserMediator(None, file_entry=file_entry)

    file_object = file_entry.GetFileObject()
    text_reader = text_parser.EncodedTextReader(file_object)
    text_reader.ReadLines()

    result = plugin.CheckRequiredFormat(parser_mediator, text_reader)
    self.assertFalse(result)

  def testProcessChromeOS(self):
    """Tests the Process function with a ChromeOS syslog file."""
    plugin = syslog.SyslogTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(
        ['syslog', 'syslog_chromeos'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 8)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'body': 'cleanup_logs: job completed',
        'data_type': 'syslog:line',
        'last_written_time': '2016-10-25T12:37:23.297265-07:00',
        'pid': 13707,
        'reporter': 'periodic_scheduler',
        'severity': 'INFO'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

  def testProcessRsyslog(self):
    """Tests the Process function with a rsyslog file."""
    plugin = syslog.SyslogTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(
        ['syslog', 'syslog_rsyslog'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 5)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'syslog:line',
        'hostname': 'localhost',
        'last_written_time': '2020-05-31T00:00:45.698463+00:00',
        'reporter': 'rsyslogd',
        'severity': None}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

  def testProcessRsyslogProtocol23(self):
    """Tests the Process function with a protocol 23 rsyslog file."""
    plugin = syslog.SyslogTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(
        ['syslog', 'syslog_rsyslog_SyslogProtocol23Format'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 9)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'syslog:line',
        'hostname': 'hostname',
        'last_written_time': '2021-03-06T04:07:38.251122+00:00',
        'reporter': 'log_tag',
        'severity': 'DEBUG'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


class TraditionalSyslogTextPluginTest(test_lib.TextPluginTestCase):
  """Tests for the traditional syslog text parser plugin."""

  def testCheckRequiredFormat(self):
    """Tests for the CheckRequiredFormat method."""
    plugin = syslog.TraditionalSyslogTextPlugin()

    # Check traditional syslog format.
    file_system_builder = fake_file_system_builder.FakeFileSystemBuilder()
    file_system_builder.AddFile('/file.txt', (
        b'Jan 22 07:52:33 myhostname.myhost.com client[30840]: INFO No new '
        b'content in \xc3\xadmynd.dd.\n'))

    file_entry = file_system_builder.file_system.GetFileEntryByPath('/file.txt')

    parser_mediator = self._CreateParserMediator(None, file_entry=file_entry)

    file_object = file_entry.GetFileObject()
    text_reader = text_parser.EncodedTextReader(file_object)
    text_reader.ReadLines()

    result = plugin.CheckRequiredFormat(parser_mediator, text_reader)
    self.assertTrue(result)

    # Check syslogkd rsyslog format.
    file_system_builder = fake_file_system_builder.FakeFileSystemBuilder()
    file_system_builder.AddFile('/file.txt', (
        b'Mar  6 04:07:28 hostname systemd[1]: Started Regular background '
        b'program processing daemon.\n'))

    file_entry = file_system_builder.file_system.GetFileEntryByPath('/file.txt')

    parser_mediator = self._CreateParserMediator(None, file_entry=file_entry)

    file_object = file_entry.GetFileObject()
    text_reader = text_parser.EncodedTextReader(file_object)
    text_reader.ReadLines()

    result = plugin.CheckRequiredFormat(parser_mediator, text_reader)
    self.assertTrue(result)

    # Check traditional rsyslog format.
    file_system_builder = fake_file_system_builder.FakeFileSystemBuilder()
    file_system_builder.AddFile('/file.txt', (
        b'Jan 22 07:54:32 myhostname.myhost.com Job `cron.daily\' '
        b'terminated\n'))

    file_entry = file_system_builder.file_system.GetFileEntryByPath('/file.txt')

    parser_mediator = self._CreateParserMediator(None, file_entry=file_entry)

    file_object = file_entry.GetFileObject()
    text_reader = text_parser.EncodedTextReader(file_object)
    text_reader.ReadLines()

    result = plugin.CheckRequiredFormat(parser_mediator, text_reader)
    self.assertTrue(result)

    # Check ChromeOS syslog format.
    file_system_builder = fake_file_system_builder.FakeFileSystemBuilder()
    file_system_builder.AddFile('/file.txt', (
        b'2016-10-25T12:37:23.297265-07:00 INFO periodic_scheduler[13707]: '
        b'cleanup_logs: job completed\n'))

    file_entry = file_system_builder.file_system.GetFileEntryByPath('/file.txt')

    parser_mediator = self._CreateParserMediator(None, file_entry=file_entry)

    file_object = file_entry.GetFileObject()
    text_reader = text_parser.EncodedTextReader(file_object)
    text_reader.ReadLines()

    result = plugin.CheckRequiredFormat(parser_mediator, text_reader)
    self.assertFalse(result)

    # Check rsyslog format.
    file_system_builder = fake_file_system_builder.FakeFileSystemBuilder()
    file_system_builder.AddFile('/file.txt', (
        b'2020-05-31T00:00:45.738158+00:00 localhost systemd[1]: Reloaded '
        b'System Logging Service.\n'))

    file_entry = file_system_builder.file_system.GetFileEntryByPath('/file.txt')

    parser_mediator = self._CreateParserMediator(None, file_entry=file_entry)

    file_object = file_entry.GetFileObject()
    text_reader = text_parser.EncodedTextReader(file_object)
    text_reader.ReadLines()

    result = plugin.CheckRequiredFormat(parser_mediator, text_reader)
    self.assertFalse(result)

    # Check protocol 23 rsyslog format.
    file_system_builder = fake_file_system_builder.FakeFileSystemBuilder()
    file_system_builder.AddFile('/file.txt', (
        b'<30>1 2021-03-06T04:07:38.265422+00:00 hostname systemd 1 - -  '
        b'Started Regular background program processing daemon.\n'))

    file_entry = file_system_builder.file_system.GetFileEntryByPath('/file.txt')

    parser_mediator = self._CreateParserMediator(None, file_entry=file_entry)

    file_object = file_entry.GetFileObject()
    text_reader = text_parser.EncodedTextReader(file_object)
    text_reader.ReadLines()

    result = plugin.CheckRequiredFormat(parser_mediator, text_reader)
    self.assertFalse(result)

    # Check non-syslog format.
    file_system_builder = fake_file_system_builder.FakeFileSystemBuilder()
    file_system_builder.AddFile('/file.txt', (
        b'gpgv: Signature made Wed Oct 22 17:40:30 2014 UTC using DSA key ID '
        b'437D05B5\n'))

    file_entry = file_system_builder.file_system.GetFileEntryByPath('/file.txt')

    parser_mediator = self._CreateParserMediator(None, file_entry=file_entry)

    file_object = file_entry.GetFileObject()
    text_reader = text_parser.EncodedTextReader(file_object)
    text_reader.ReadLines()

    result = plugin.CheckRequiredFormat(parser_mediator, text_reader)
    self.assertFalse(result)

  def testProcess(self):
    """Tests the Process function."""
    plugin = syslog.TraditionalSyslogTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(['syslog', 'syslog'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 16)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'body': 'INFO No new content in ímynd.dd.',
        'data_type': 'syslog:line',
        'hostname': 'myhostname.myhost.com',
        'last_written_time': '0000-01-22T07:52:33',
        'pid': 30840,
        'reporter': 'client',
        'severity': None}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

    # Check if year is incremented.
    expected_event_values = {
        'body': 'This syslog message has a fractional value for seconds.',
        'data_type': 'syslog:line',
        'last_written_time': '0001-03-23T23:01:18',
        'reporter': 'somrandomexe',
        'severity': None}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 9)
    self.CheckEventData(event_data, expected_event_values)

  def testProcessCron(self):
    """Tests the Process function with a cron syslog file."""
    plugin = syslog.TraditionalSyslogTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(
        ['syslog', 'syslog_cron.log'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 9)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'command': 'sleep $(( 1 * 60 )); touch /tmp/afile.txt',
        'data_type': 'syslog:cron:task_run',
        'last_written_time': '0000-03-11T19:26:39',
        'username': 'root'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 1)
    self.CheckEventData(event_data, expected_event_values)

  def testProcessDarwin(self):
    """Tests the Process function with a Darwin syslog file."""
    plugin = syslog.TraditionalSyslogTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(
        ['syslog', 'syslog_osx'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 2)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'syslog:line',
        'hostname': 'osx-machine',
        'last_written_time': '0000-03-11T19:26:39',
        'reporter': 'kernel',
        'severity': None}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

  def testProcessRsyslogSysklogd(self):
    """Tests the Process function with a syslogkd rsyslog file."""
    plugin = syslog.TraditionalSyslogTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(
        ['syslog', 'syslog_rsyslog_SysklogdFileFormat'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 9)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'syslog:line',
        'hostname': 'hostname',
        'last_written_time': '0000-03-06T04:07:28',
        'reporter': 'log_tag',
        'severity': None}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

  def testProcessRsyslogTraditional(self):
    """Tests the Process function with a traditional rsyslog file."""
    plugin = syslog.TraditionalSyslogTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(
        ['syslog', 'syslog_rsyslog_traditional'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 8)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'syslog:line',
        'hostname': 'myhostname.myhost.com',
        'last_written_time': '0000-01-22T07:54:32',
        'reporter': 'Job',
        'severity': None}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

  def testProcessSshd(self):
    """Tests the Process function with a sshd syslog file."""
    plugin = syslog.TraditionalSyslogTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(
        ['syslog', 'syslog_ssh.log'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 9)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'syslog:line',
        'last_written_time': '0000-03-11T00:00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

    expected_event_values = {
        'body': (
            'Accepted publickey for plaso from 192.168.0.1 port 59229 ssh2: '
            'RSA 00:aa:bb:cc:dd:ee:ff:11:22:33:44:55:66:77:88:99'),
        'data_type': 'syslog:ssh:login',
        'fingerprint': 'RSA 00:aa:bb:cc:dd:ee:ff:11:22:33:44:55:66:77:88:99',
        'ip_address': '192.168.0.1',
        'last_written_time': '0000-03-11T19:26:39'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 1)
    self.CheckEventData(event_data, expected_event_values)

    expected_event_values = {
        'data_type': 'syslog:ssh:failed_connection',
        'ip_address': '001:db8:a0b:12f0::1',
        'last_written_time': '0000-03-11T22:55:30',
        'port': '8759'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 3)
    self.CheckEventData(event_data, expected_event_values)

    expected_event_values = {
        'data_type': 'syslog:ssh:opened_connection',
        'ip_address': '188.124.3.41',
        'last_written_time': '0000-03-11T22:55:31'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 4)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the BagMRU Windows Registry plugin."""

import unittest

from plaso.parsers.winreg_plugins import bagmru

from tests.parsers.winreg_plugins import test_lib


class TestBagMRUWindowsRegistryPlugin(test_lib.RegistryPluginTestCase):
  """Tests for the BagMRU plugin."""

  def testFilters(self):
    """Tests the FILTERS class attribute."""
    plugin = bagmru.BagMRUWindowsRegistryPlugin()

    self._AssertFiltersOnKeyPath(plugin, 'HKEY_CURRENT_USER', (
        'Software\\Microsoft\\Windows\\Shell\\BagMRU'))

    self._AssertFiltersOnKeyPath(plugin, 'HKEY_CURRENT_USER', (
        'Software\\Microsoft\\Windows\\ShellNoRoam\\BagMRU'))

    self._AssertFiltersOnKeyPath(
        plugin, 'HKEY_CURRENT_USER\\Software\\Classes',
        'Local Settings\\Software\\Microsoft\\Windows\\Shell\\BagMRU')

    self._AssertFiltersOnKeyPath(
        plugin, 'HKEY_CURRENT_USER\\Software\\Classes',
        'Local Settings\\Software\\Microsoft\\Windows\\ShellNoRoam\\BagMRU')

    self._AssertFiltersOnKeyPath(plugin, 'HKEY_CURRENT_USER', (
        'Local Settings\\Software\\Microsoft\\Windows\\Shell\\BagMRU'))

    self._AssertFiltersOnKeyPath(plugin, 'HKEY_CURRENT_USER', (
        'Local Settings\\Software\\Microsoft\\Windows\\ShellNoRoam\\BagMRU'))

    self._AssertNotFiltersOnKeyPath(plugin, 'HKEY_CURRENT_USER', 'Bogus')

  def testProcess(self):
    """Tests the Process function."""
    plugin = bagmru.BagMRUWindowsRegistryPlugin()
    test_file_entry = self._GetTestFileEntry(['NTUSER.DAT'])
    key_path = (
        'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\ShellNoRoam\\BagMRU')

    win_registry = self._GetWinRegistryFromFileEntry(test_file_entry)
    registry_key = win_registry.GetKeyByPath(key_path)
    storage_writer = self._ParseKeyWithPlugin(
        registry_key, plugin, file_entry=test_file_entry)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 9)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'windows:registry:bagmru',
        'entries': 'Index: 1 [MRU Value 0]: Shell item path: <My Computer>',
        'key_path': key_path,
        'last_written_time': '2009-08-04T15:19:16.9977500+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

    expected_event_values = {
        'data_type': 'windows:registry:bagmru',
        'entries': (
            'Index: 1 [MRU Value 0]: Shell item path: <My Computer> C:\\\\'),
        'key_path': f'{key_path:s}\\0',
        'last_written_time': '2009-08-04T15:19:10.6696250+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 1)
    self.CheckEventData(event_data, expected_event_values)

    expected_event_values = {
        'data_type': 'windows:registry:bagmru',
        'entries': None,
        'key_path': f'{key_path:s}\\0\\0\\0\\0\\0',
        'last_written_time': '2009-08-04T15:19:16.9977500+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 8)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Microsoft Internet Explorer (MSIE) Cache Files (CF) parser."""

import unittest

from plaso.parsers import msiecf

from tests.parsers import test_lib


class MSIECFParserTest(test_lib.ParserTestCase):
  """Tests for the MSIE Cache Files (MSIECF) parser."""

  def testParse(self):
    """Tests the Parse function."""
    parser = msiecf.MSIECFParser()
    storage_writer = self._ParseFile(['index.dat'], parser)

    # MSIE Cache File information:
    #   Version                         : 5.2
    #   File size                       : 32768 bytes
    #   Number of items                 : 7
    #   Number of recovered items       : 11

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 18)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    # Record type             : URL
    # Offset range            : 21376 - 21632 (256)
    # Location                : Visited: testing@http://www.trafficfusionx.com
    #                           /download/tfscrn2/funnycats.exe
    # Primary time            : Jun 23, 2011 18:02:10.066000000
    # Secondary time          : Jun 23, 2011 18:02:10.066000000
    # Expiration time         : Jun 29, 2011 17:55:02
    # Last checked time       : Jun 23, 2011 18:02:12
    # Cache directory index   : -2 (0xfe)

    expected_event_values = {
        'access_time': None,
        'cache_directory_index': -2,
        'cached_file_size': 0,
        'creation_time': None,
        'data_type': 'msiecf:url',
        'expiration_time': '2011-06-29T17:55:02+00:00',
        'last_visited_time': '2011-06-23T18:02:10.0660000+00:00',
        'modification_time': None,
        'number_of_hits': 6,
        'offset': 21376,
        'primary_time': None,
        'recovered': False,
        'secondary_time': '2011-06-23T18:02:10.0660000+00:00',
        'synchronization_time': '2011-06-23T18:02:12+00:00',
        'url': (
            'Visited: testing@http://www.trafficfusionx.com/download/tfscrn2'
            '/funnycats.exe')}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 2)
    self.CheckEventData(event_data, expected_event_values)

  def testParseLeakAndRedirect(self):
    """Tests the Parse function with leak and redirected records."""
    parser = msiecf.MSIECFParser()
    storage_writer = self._ParseFile(['nfury_index.dat'], parser)

    # MSIE Cache File information:
    #   Version                         : 5.2
    #   File size                       : 491520 bytes
    #   Number of items                 : 1027
    #   Number of recovered items       : 8

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1035)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'access_time': '2010-11-10T07:54:30.0920000+00:00',
        'cache_directory_index': 0,
        'cache_directory_name': 'R6QWCVX4',
        'cached_file_size': 4286,
        'cached_filename': 'favicon[1].ico',
        'creation_time': None,
        'data_type': 'msiecf:url',
        'expiration_time': '2011-09-13T21:36:18+00:00',
        'last_visited_time': None,
        'modification_time': '2010-08-10T00:03:00.0000000+00:00',
        'http_headers': (
            'HTTP/1.1 200 OK\r\n'
            'Content-Type: image/x-icon\r\n'
            'ETag: "0922651f38cb1:0",\r\n'
            'X-Powered-By: ASP.NET\r\n'
            'P3P: CP="BUS CUR CONo FIN IVDo ONL OUR PHY SAMo TELo"\r\n'
            'Content-Length: 4286\r\n'
            '\r\n'
            '~U:nfury\r\n'),
        'number_of_hits': 1,
        'offset': 24576,
        'primary_time': None,
        'recovered': False,
        'secondary_time': None,
        'synchronization_time': '2010-11-10T07:54:32+00:00',
        'url': 'http://col.stc.s-msn.com/br/gbl/lg/csl/favicon.ico'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

    expected_event_values = {
        'cache_directory_index': 1,
        'cache_directory_name': 'VUQHQA73',
        'cached_file_size': 1966,
        'cached_filename': 'ADSAdClient31[1].htm',
        'data_type': 'msiecf:leak',
        'offset': 26368,
        'recovered': False}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 4)
    self.CheckEventData(event_data, expected_event_values)

    expected_event_values = {
        'data_type': 'msiecf:redirected',
        'offset': 26880,
        'recovered': False,
        'url': (
            'http://ad.doubleclick.net/ad/N2724.Meebo/B5343067.13;'
            'sz=1x1;pc=[TPAS_ID];ord=2642102')}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 6)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""SQLite parser plugin for Windows 10 push notification database files."""

from plaso.containers import events
from plaso.parsers import sqlite
from plaso.parsers.sqlite_plugins import interface


class WindowsPushNotificationEventData(events.EventData):
  """Windows push notification event data.

  Attributes:
    arrival_time (dfdatetime.DateTimeValues): date and time the push
        notification was received.
    boot_time (dfdatetime.DateTimeValues): date and time the of the last boot.
    expiration_time (dfdatetime.DateTimeValues): date and time the push
        notification expires.
    handler_identifier (str): identifier of the corresponding notification
        handler.
    notification_type (str): notification type.
    payload (dfdatetime.DateTimeValues): payload.
  """

  DATA_TYPE = 'windows:wpndatabase:notification'

  def __init__(self):
    """Initialize event data."""
    super(WindowsPushNotificationEventData, self).__init__(
        data_type=self.DATA_TYPE)
    self.arrival_time = None
    self.boot_time = None
    self.expiration_time = None
    self.handler_identifier = None
    self.notification_type = None
    self.payload = None


class WindowsPushNotificationHandlerEventData(events.EventData):
  """Windows push notification handler event data.

  Attributes:
    creation_time (dfdatetime.DateTimeValues): date and time the push
        notification handler was created.
    handler_type (str): handler type.
    identifier (str): identifier.
    modification_time (dfdatetime.DateTimeValues): date and time the push
        notification handler was last modified.
    service_identifier (str): Windows Push Notification Service (WNS)
        identifier.
  """

  DATA_TYPE = 'windows:wpndatabase:notification_handler'

  def __init__(self):
    """Initialize event data."""
    super(WindowsPushNotificationHandlerEventData, self).__init__(
        data_type=self.DATA_TYPE)
    self.creation_time = None
    self.handler_type = None
    self.identifier = None
    self.modification_time = None
    self.service_identifier = None


class WindowsPushNotificationPlugin(interface.SQLitePlugin):
  """SQLite parser plugin for Windows 10 push notification database files.

  The Windows 10 push notification database file is typically stored in:
  %APPDATA%\\Local\\Microsoft\\Windows\\Notifications\\wpndatabase.db
  """

  NAME = 'windows_push_notification'
  DATA_FORMAT = (
      'Windows 10 push notification SQLite database (wpndatabase.db) file')

  REQUIRED_STRUCTURE = {
      'Metadata': frozenset([
          'Key', 'Value']),
      'Notification': frozenset([
          'ActivityId', 'ArrivalTime', 'BootId', 'DataVersion',
          'ExpiresOnReboot', 'ExpiryTime', 'HandlerId', 'Group', 'Id',
          'Order', 'Payload', 'PayloadType', 'Tag', 'Type']),
      'NotificationHandler': frozenset([
          'ContainerSid', 'CreatedTime', 'HandlerType', 'ModifiedTime',
          'ParentId', 'PrimaryId', 'RecordId', 'SystemDataPropertySet',
          'WNFEventName', 'WNSId'])}

  QUERIES = [
      (('SELECT RecordId, PrimaryId, WNSId, HandlerType, WNFEventName, '
        'SystemDataPropertySet, CreatedTime, ModifiedTime, ParentId, '
        'ContainerSid FROM NotificationHandler'),
       'ParseNotificationHandlerRow'),
      # Note that Order and Group must be escaped other they are interpreted
      # as SQL keywords.
      (('SELECT "Order", Id, ActivityId, Type, Payload, Tag, "Group", '
        'ExpiryTime, ArrivalTime, DataVersion, PayloadType, BootId, '
        'ExpiresOnReboot, PrimaryId FROM Notification, NotificationHandler '
        'WHERE Notification.HandlerId == NotificationHandler.RecordId'),
       'ParseNotificationRow')]

  SCHEMAS = [{
      'HandlerAssets': (
          'CREATE TABLE [HandlerAssets]( [HandlerId] INTEGER '
          'CONSTRAINT[AssetsToHandler] '
          'REFERENCES[NotificationHandler]([RecordId]) ON DELETE CASCADE ON '
          'UPDATE CASCADE, [AssetKey] TEXT NOT NULL, [AssetValue] TEXT, '
          'CONSTRAINT[] PRIMARY KEY([AssetKey], [HandlerId]) ON CONFLICT '
          'REPLACE)'),
      'HandlerSettings': (
          'CREATE TABLE [HandlerSettings]( [HandlerId] INTEGER '
          'CONSTRAINT[SettingsToHandler] '
          'REFERENCES[NotificationHandler]([RecordId]) ON DELETE CASCADE ON '
          'UPDATE CASCADE, [SettingKey] TEXT NOT NULL, [Value] INT, '
          'CONSTRAINT[] PRIMARY KEY([SettingKey], [HandlerId]) ON CONFLICT '
          'REPLACE)'),
      'Metadata': (
          'CREATE TABLE [Metadata]( [Key] TEXT, [Value] INT64, CONSTRAINT[] '
          'PRIMARY KEY([Key]) ON CONFLICT REPLACE)'),
      'Notification': (
          'CREATE TABLE [Notification]( [Order] INTEGER NOT NULL PRIMARY KEY, '
          '[Id] INTEGER NOT NULL, [HandlerId] INTEGER '
          'CONSTRAINT[NotificationToHandler] '
          'REFERENCES[NotificationHandler]([RecordId]) ON DELETE CASCADE ON '
          'UPDATE CASCADE, [ActivityId] GUID,[Type] TEXT NOT NULL, [Payload] '
          'BLOB, [Tag] TEXT, [Group] TEXT, [ExpiryTime] INT64, [ArrivalTime] '
          'INT64, [DataVersion] INT64 DEFAULT \'0\', [PayloadType] TEXT NOT '
          'NULL, [BootId] INT64 DEFAULT \'0\', [ExpiresOnReboot] BOOLEAN '
          'DEFAULT \'FALSE\', UNIQUE([Id]) ON CONFLICT REPLACE)'),
      'NotificationData': (
          'CREATE TABLE [NotificationData]( [NotificationId] INTEGER '
          'CONSTRAINT[DataToNotification] REFERENCES[Notification]([Id]) ON '
          'DELETE CASCADE ON UPDATE CASCADE, [Key] TEXT NOT NULL, [Value] '
          'TEXT, CONSTRAINT[] PRIMARY KEY([Key], [NotificationId]) ON '
          'CONFLICT REPLACE)'),
      'NotificationHandler': (
          'CREATE TABLE [NotificationHandler] ( [RecordId] INTEGER PRIMARY '
          'KEY, [PrimaryId] TEXT NOT NULL COLLATE NOCASE, [WNSId] TEXT '
          'COLLATE NOCASE, [HandlerType] TEXT, [WNFEventName] INT64, '
          '[SystemDataPropertySet] BLOB, [CreatedTime] DATETIME, '
          '[ModifiedTime] DATETIME, [ParentId] TEXT COLLATE NOCASE, '
          '[ContainerSid] TEXT COLLATE NOCASE)'),
      'TransientTable': (
          'CREATE TABLE [TransientTable]( [OfflineCacheCount] INTEGER, '
          '[NotificationId] INTEGER CONSTRAINT[TransientToNotification] '
          'REFERENCES[Notification]([Id]) ON DELETE CASCADE ON UPDATE '
          'CASCADE, [OfflineBundleId] TEXT, [ServerCacheRollover] BOOLEAN '
          'DEFAULT \'FALSE\', [CrossDeviceMatchId] TEXT, [SuppressPopup] '
          'BOOLEAN DEFAULT \'FALSE\', [IsMirroringDisabled] BOOLEAN DEFAULT '
          '\'FALSE\', [RecurrenceId] GUID, [MessageId] GUID, [Priority] '
          'INTEGER NOT NULL, [CV] TEXT)'),
      'WNSPushChannel': (
          'CREATE TABLE [WNSPushChannel]( [ChannelId] TEXT NOT NULL, '
          '[HandlerId] INTEGER REFERENCES[NotificationHandler]([RecordId]) ON '
          'DELETE CASCADE ON UPDATE CASCADE, [Uri] TEXT, [ExpiryTime] INT64, '
          '[CreatedTime] INT64, [DeviceVersion] INT64 DEFAULT \'0\', '
          'CONSTRAINT[] PRIMARY KEY([ChannelId]) ON CONFLICT REPLACE)')}]

  def ParseNotificationHandlerRow(
      self, parser_mediator, query, row, **unused_kwargs):
    """Parses a row of the NotificationHandler table.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      query (str): query that created the row.
      row (sqlite3.Row): row.
    """
    query_hash = hash(query)

    event_data = WindowsPushNotificationHandlerEventData()
    event_data.creation_time = self._GetDateTimeStringRowValue(
        query_hash, row, 'CreatedTime')
    event_data.handler_type = self._GetRowValue(query_hash, row, 'HandlerType')
    event_data.identifier = self._GetRowValue(query_hash, row, 'PrimaryId')
    event_data.modification_time = self._GetDateTimeStringRowValue(
        query_hash, row, 'ModifiedTime')
    event_data.service_identifier = self._GetRowValue(query_hash, row, 'WNSId')

    # TODO: add support for WNFEventName

    parser_mediator.ProduceEventData(event_data)

  def ParseNotificationRow(
      self, parser_mediator, query, row, **unused_kwargs):
    """Parses a row of the Notification table.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      query (str): query that created the row.
      row (sqlite3.Row): row.
    """
    query_hash = hash(query)

    payload = None
    payload_type = self._GetRowValue(query_hash, row, 'PayloadType')

    if payload_type.lower() == 'xml':
      payload = self._GetRowValue(query_hash, row, 'Payload')
      payload = payload.decode('utf-8')
      # TODO: parse payload
    else:
      parser_mediator.ProduceExtractionWarning(
          f'unsupported payload type: {payload_type:s}')

    event_data = WindowsPushNotificationEventData()
    event_data.arrival_time = self._GeFiletimeRowValue(
        query_hash, row, 'ArrivalTime')
    event_data.boot_time = self._GeFiletimeRowValue(
        query_hash, row, 'BootId')
    event_data.expiration_time = self._GeFiletimeRowValue(
        query_hash, row, 'ExpiryTime')
    event_data.handler_identifier = self._GetRowValue(
        query_hash, row, 'PrimaryId')
    event_data.notification_type = self._GetRowValue(query_hash, row, 'Type')
    event_data.payload = payload

    parser_mediator.ProduceEventData(event_data)


sqlite.SQLiteParser.RegisterPlugin(WindowsPushNotificationPlugin)



# -*- coding: utf-8 -*-
"""Plist parser plugin for MacOS TimeMachine plist files."""

import os

import pytz

from dfdatetime import time_elements as dfdatetime_time_elements

from plaso.containers import events
from plaso.lib import dtfabric_helper
from plaso.lib import errors
from plaso.parsers import plist
from plaso.parsers.plist_plugins import interface


class MacOSTimeMachineBackupEventData(events.EventData):
  """MacOS TimeMachine backup event data.

  Attributes:
    backup_alias (str): alias of the backup.
    destination_identifier (str): identifier of the destination volume.
    snapshot_times (list[dfdatetime.DateTimeValues]): dates and times of
        the creation of backup snaphots.
  """

  DATA_TYPE = 'macos:time_machine:backup'

  def __init__(self):
    """Initializes event data."""
    super(MacOSTimeMachineBackupEventData, self).__init__(
        data_type=self.DATA_TYPE)
    self.backup_alias = None
    self.destination_identifier = None
    self.snapshot_times = None


class MacOSTimeMachinePlistPlugin(
    interface.PlistPlugin, dtfabric_helper.DtFabricHelper):
  """Plist parser plugin for MacOS TimeMachine plist files.

  Further details about the extracted fields:
    DestinationID:
      remote UUID hard disk where the backup is done.

    BackupAlias:
      structure that contains the extra information from the destinationID.

    SnapshotDates:
      list of the backup dates.
  """

  NAME = 'time_machine'
  DATA_FORMAT = 'MacOS TimeMachine plist file'

  PLIST_PATH_FILTERS = frozenset([
      interface.PlistPathFilter('com.apple.TimeMachine.plist')])

  PLIST_KEYS = frozenset(['Destinations', 'RootVolumeUUID'])

  _DEFINITION_FILE = os.path.join(
      os.path.dirname(__file__), 'time_machine.yaml')

  def __init__(self):
    """Initializes a plist parser plugin."""
    super(MacOSTimeMachinePlistPlugin, self).__init__()
    self._backup_alias_map = self._GetDataTypeMap('timemachine_backup_alias')

  # pylint: disable=arguments-differ
  def _ParsePlist(self, parser_mediator, match=None, **unused_kwargs):
    """Extracts relevant MacOS TimeMachine entries.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      match (Optional[dict[str: object]]): keys extracted from PLIST_KEYS.
    """
    for destination in match.get('Destinations', []):
      backup_alias_string = None
      snapshot_times = []

      backup_alias_data = destination.get('BackupAlias', None)
      if backup_alias_data:
        try:
          backup_alias = self._ReadStructureFromByteStream(
              backup_alias_data, 0, self._backup_alias_map)
          backup_alias_string = backup_alias.string

        except (ValueError, TypeError, errors.ParseError) as exception:
          parser_mediator.ProduceExtractionWarning(
              'unable to parse backup alias value with error: {0!s}'.format(
                  exception))

      for datetime_value in destination.get('SnapshotDates', []):
        # dfDateTime relies on the time zone but since plistlib does not set
        # one.
        datetime_value = datetime_value.replace(tzinfo=pytz.UTC)

        date_time = dfdatetime_time_elements.TimeElementsInMicroseconds()
        date_time.CopyFromDatetime(datetime_value)
        snapshot_times.append(date_time)

      event_data = MacOSTimeMachineBackupEventData()
      event_data.backup_alias = backup_alias_string
      event_data.destination_identifier = destination.get('DestinationID', None)
      event_data.snapshot_times = snapshot_times or None

      parser_mediator.ProduceEventData(event_data)


plist.PlistParser.RegisterPlugin(MacOSTimeMachinePlistPlugin)



# -*- coding: utf-8 -*-
"""Windows PE/COFF resource file helper."""

import re


class WindowsResourceFileHelper(object):
  """Windows PE/COFF resource file helper."""

  # Message string specifiers that are considered white space.
  _MESSAGE_STRING_WHITE_SPACE_SPECIFIER_RE = re.compile(r'(%[0b]|[\r\n])')

  # Message string specifiers that expand to text.
  _MESSAGE_STRING_TEXT_SPECIFIER_RE = re.compile(r'%([ .!%nrt])')

  # Curly brackets in a message string.
  _MESSAGE_STRING_CURLY_BRACKETS = re.compile(r'([\{\}])')

  # Message string specifiers that expand to a variable place holder.
  _MESSAGE_STRING_PLACE_HOLDER_SPECIFIER_RE = re.compile(
      r'%([1-9][0-9]?)[!]?[s]?[!]?')

  @classmethod
  def _MessageStringPlaceHolderSpecifierReplacer(cls, match_object):
    """Replaces message string place holders into Python format() style.

    Args:
      match_object (re.Match): regular expression match object.

    Returns:
      str: message string with Python format() style place holders.
    """
    expanded_groups = []

    for group in match_object.groups():
      try:
        place_holder_number = int(group, 10) - 1
        expanded_group = '{{{0:d}:s}}'.format(place_holder_number)
      except ValueError:
        expanded_group = group

      expanded_groups.append(expanded_group)

    return ''.join(expanded_groups)

  @classmethod
  def FormatMessageStringInPEP3101(cls, message_string):
    """Formats a message string in Python format() (PEP 3101) style.

    Args:
      message_string (str): message string.

    Returns:
      str: message string in Python format() (PEP 3101) style.
    """
    if not message_string:
      return None

    message_string = message_string.rstrip('\0')
    message_string = cls._MESSAGE_STRING_WHITE_SPACE_SPECIFIER_RE.sub(
        r'', message_string)
    message_string = cls._MESSAGE_STRING_TEXT_SPECIFIER_RE.sub(
        r'\\\1', message_string)
    message_string = cls._MESSAGE_STRING_CURLY_BRACKETS.sub(
        r'\1\1', message_string)
    return cls._MESSAGE_STRING_PLACE_HOLDER_SPECIFIER_RE.sub(
        cls._MessageStringPlaceHolderSpecifierReplacer, message_string)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the iOS lockdown daemon log files text parser plugin."""

import unittest

from plaso.parsers.text_plugins import ios_lockdownd

from tests.parsers.text_plugins import test_lib


class IOSLockdowndLogTextPluginTest(test_lib.TextPluginTestCase):
  """Tests for the iOS lockdown daemon log files text parser plugin."""

  def testProcess(self):
    """Tests the Process function."""
    plugin = ios_lockdownd.IOSLockdowndLogTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(
        ['ios_lockdownd.log'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 153)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'body': (
            'handle_get_value: AMPDevicesAgent attempting to get '
            '[InternationalMobileSubscriberIdentity2]'),
        'data_type': 'ios:lockdownd_log:entry',
        'process_identifier': 69,
        'written_time': '2021-10-13T07:57:42.869324+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 6)
    self.CheckEventData(event_data, expected_event_values)

    expected_event_values = {
        'body': (
            'spawn_xpc_service_block_invoke: description of xpc reply: '
            '<dictionary: 0x2029c5070> { count = 1, transaction: 0, '
            'voucher = 0x0, contents = "XPCErrorDescription" => <string: '
            '0x2029c5230> { length = 22, contents = "Connection '
            'interrupted" } }'),
        'data_type': 'ios:lockdownd_log:entry',
        'process_identifier': 69,
        'written_time': '2021-10-13T07:57:42.950704+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 99)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Network Drives Windows Registry plugin."""

import unittest

from dfdatetime import filetime as dfdatetime_filetime
from dfwinreg import definitions as dfwinreg_definitions
from dfwinreg import fake as dfwinreg_fake

from plaso.parsers.winreg_plugins import network_drives

from tests.parsers.winreg_plugins import test_lib


class NetworkDrivesPluginTest(test_lib.RegistryPluginTestCase):
  """Tests for the Network Drives Windows Registry plugin."""

  def _CreateTestKey(self):
    """Creates Registry keys and values for testing.

    Returns:
      dfwinreg.WinRegistryKey: Windows Registry key.
    """
    filetime = dfdatetime_filetime.Filetime()
    filetime.CopyFromDateTimeString('2013-01-30 10:47:57')
    registry_key = dfwinreg_fake.FakeWinRegistryKey(
        'Network', key_path_prefix='HKEY_CURRENT_USER',
        last_written_time=filetime.timestamp, offset=153,
        relative_key_path='Network')

    # Setup H drive.
    h_key_name = 'H'
    h_key = dfwinreg_fake.FakeWinRegistryKey(
        h_key_name, last_written_time=filetime.timestamp)
    registry_key.AddSubkey(h_key_name, h_key)

    value_data = b'\x00\x00\x00\x01'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'ConnectionType', data=value_data,
        data_type=dfwinreg_definitions.REG_DWORD_BIG_ENDIAN)
    h_key.AddValue(registry_value)

    value_data = b'\x00\x00\x00\x04'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'DeferFlags', data=value_data,
        data_type=dfwinreg_definitions.REG_DWORD_BIG_ENDIAN)
    h_key.AddValue(registry_value)

    value_data = b'\x00\x00\x00\x01'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'ProviderFlags', data=value_data,
        data_type=dfwinreg_definitions.REG_DWORD_BIG_ENDIAN)
    h_key.AddValue(registry_value)

    value_data = 'Microsoft Windows Network'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'ProviderName', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    h_key.AddValue(registry_value)

    value_data = b'\x00\x02\x00\x00'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'ProviderType', data=value_data,
        data_type=dfwinreg_definitions.REG_DWORD_BIG_ENDIAN)
    h_key.AddValue(registry_value)

    value_data = '\\\\acme.local\\Shares\\User_Data\\John.Doe'.encode(
        'utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'RemotePath', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    h_key.AddValue(registry_value)

    value_data = b'\x00\x00\x00\x00'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'UserName', data=value_data,
        data_type=dfwinreg_definitions.REG_DWORD_BIG_ENDIAN)
    h_key.AddValue(registry_value)

    # Setup Z drive.
    z_key_name = 'Z'
    z_key = dfwinreg_fake.FakeWinRegistryKey(
        z_key_name, last_written_time=filetime.timestamp)
    registry_key.AddSubkey(z_key_name, z_key)

    value_data = b'\x00\x00\x00\x01'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'ConnectionType', data=value_data,
        data_type=dfwinreg_definitions.REG_DWORD_BIG_ENDIAN)
    z_key.AddValue(registry_value)

    value_data = b'\x00\x00\x00\x04'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'DeferFlags', data=value_data,
        data_type=dfwinreg_definitions.REG_DWORD_BIG_ENDIAN)
    z_key.AddValue(registry_value)

    value_data = b'\x00\x00\x00\x01'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'ProviderFlags', data=value_data,
        data_type=dfwinreg_definitions.REG_DWORD_BIG_ENDIAN)
    z_key.AddValue(registry_value)

    value_data = 'Microsoft Windows Network'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'ProviderName', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    z_key.AddValue(registry_value)

    value_data = b'\x00\x02\x00\x00'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'ProviderType', data=value_data,
        data_type=dfwinreg_definitions.REG_DWORD_BIG_ENDIAN)
    z_key.AddValue(registry_value)

    value_data = '\\\\secret_computer\\Media'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'RemotePath', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    z_key.AddValue(registry_value)

    value_data = b'\x00\x00\x00\x00'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'UserName', data=value_data,
        data_type=dfwinreg_definitions.REG_DWORD_BIG_ENDIAN)
    z_key.AddValue(registry_value)

    return registry_key

  def testFilters(self):
    """Tests the FILTERS class attribute."""
    plugin = network_drives.NetworkDrivesPlugin()

    self._AssertFiltersOnKeyPath(plugin, 'HKEY_CURRENT_USER', 'Network')

    self._AssertNotFiltersOnKeyPath(plugin, 'HKEY_CURRENT_USER', 'Bogus')

  def testProcess(self):
    """Tests the Process function on created key."""
    registry_key = self._CreateTestKey()

    plugin = network_drives.NetworkDrivesPlugin()
    storage_writer = self._ParseKeyWithPlugin(registry_key, plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 2)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'windows:registry:network_drive',
        'drive_letter': 'H',
        'key_path': 'HKEY_CURRENT_USER\\Network',
        'last_written_time': '2013-01-30T10:47:57.0000000+00:00',
        'server_name': 'acme.local',
        'share_name': '\\Shares\\User_Data\\John.Doe'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()
